##############################################################################
# cmake configuration file for MarlinTPC
#
# requires:
#   MacroCheckPackageLibs.cmake for checking package libraries
#
# returns following variables:
#
#   MarlinTPC_FOUND      : set to TRUE if MarlinTPC found
#       if FIND_PACKAGE called with REQUIRED and COMPONENTS arguments
#       MarlinTPC_FOUND is only set to TRUE if ALL components are also found
#       if REQUIRED is NOT set components may or may not be available
#
#   MarlinTPC_ROOT       : path to this MarlinTPC installation
#   MarlinTPC_VERSION    : package version
#   MarlinTPC_LIBRARIES  : list of MarlinTPC libraries (NOT including COMPONENTS)
#   MarlinTPC_INCLUDE_DIRS  : list of paths to be used with INCLUDE_DIRECTORIES
#   MarlinTPC_LIBRARY_DIRS  : list of paths to be used with LINK_DIRECTORIES
#   MarlinTPC_COMPONENT_LIBRARIES      : list of MarlinTPC component libraries
#   MarlinTPC_${COMPONENT}_FOUND       : set to TRUE or FALSE for each library
#   MarlinTPC_${COMPONENT}_LIBRARY     : path to individual libraries
#   MarlinTPC_${COMPONENT}_LIB_DEPENDS : individual library dependencies
#
# @author Jan Engels, Desy
##############################################################################


SET( MarlinTPC_ROOT "/proj/cdc/soft/MarlinTPC/trunk" )
SET( MarlinTPC_VERSION "0.0.0" )



# do not store find results in cache
SET( MarlinTPC_INCLUDE_DIRS MarlinTPC_INCLUDE_DIRS-NOTFOUND )
MARK_AS_ADVANCED( MarlinTPC_INCLUDE_DIRS )

FIND_PATH( MarlinTPC_INCLUDE_DIRS
    NAMES marlintpc/TPCVoxel.h
    PATHS "${MarlinTPC_ROOT}/include"
    NO_DEFAULT_PATH
)



# ---------- libraries --------------------------------------------------------
INCLUDE( "/proj/soft/ilcsoft/v01-16/ilcutil/v01-00/cmakemodules/MacroCheckPackageLibs.cmake" )

# only standard libraries should be passed as arguments to CHECK_PACKAGE_LIBS
# additional components are set by cmake in variable PKG_FIND_COMPONENTS
# first argument should be the package name
CHECK_PACKAGE_LIBS( MarlinTPC MarlinTPC )



# ---------- libraries dependencies -------------------------------------------
# this sets MarlinTPC_${COMPONENT}_LIB_DEPENDS variables
INCLUDE( "${MarlinTPC_ROOT}/lib/cmake/MarlinTPCLibDeps.cmake" )



# ---------- final checking ---------------------------------------------------
INCLUDE( FindPackageHandleStandardArgs )
# set MarlinTPC_FOUND to TRUE if all listed variables are TRUE and not empty
# MarlinTPC_COMPONENT_VARIABLES will be set if FIND_PACKAGE is called with REQUIRED argument
FIND_PACKAGE_HANDLE_STANDARD_ARGS( MarlinTPC DEFAULT_MSG MarlinTPC_ROOT MarlinTPC_INCLUDE_DIRS MarlinTPC_LIBRARIES )

SET( MarlinTPC_FOUND ${MARLINTPC_FOUND} )

