#include "CutApplicationProcessor.h"

using namespace lcio ;
using namespace marlin ;

namespace marlintpc{

CutApplicationProcessor aCutApplicationProcessor ;
    

CutApplicationProcessor::CutApplicationProcessor() : Processor("CutApplicationProcessor") 
{

	/* Description of the processor which will be displayed in the
	 * steering file automatically generated by 
	 * marlin (use MarlinTPC -x )
	 */
	_description = "CutApplicationProcessor applies cuts on all parameters for further analysis";
	
	
	/* register steering parameters: name, description, class-variable, default value
	 * the type will by definded by the type of the default value
	 * string, double, float and int are possible
	 * plaese note to get a float you must cast in C++: (float) 1.3 
	 */
	registerInputCollection( LCIO::TRACK,
  				   "InputCollectionName" , 
				   "Name of the input collection"  ,
				   _inputColName ,
				    std::string("TPCTracks") ) ;

	registerOutputCollection( LCIO::TRACK,
				   "OutputCollectionName" , 
				   "Name of the output collection"  ,
				   _outputColName ,
				   std::string("TPCTracksCut") ) ;
	
	registerProcessorParameter( "SetOutputTransient" , 
				    "if not 0 the output collection is set transient, default 0"  ,
				    _outputIsTransient,
				    0);	

        registerOptionalParameter( "chipHeight" , 
				    "chip height in mm"  ,
				    _chipHeight,
				    (double) 14.08 ) ;	

        registerOptionalParameter( "chipLength" , 
				    "chip length in mm"  ,
				    _chipLength,
				    (double) 14.08 ) ;

	registerOptionalParameter( "lowCutNTracks" , 
				    "lower cut on number of tracks"  ,
				    _lowcutNTracks ,
				    (int) 0. ) ;

	registerOptionalParameter( "highCutNTracks" , 
				    "upper cut on number of tracks"  ,
				    _highcutNTracks ,
				    (int) 10. ) ;

	registerOptionalParameter( "lowCutNHitsOnTrack" , 
				    "lower cut on number of hits on track"  ,
				    _lowcutNHitsOnTrack ,
				    (int) 0. ) ;

	registerOptionalParameter( "highCutNHitsOnTrack" , 
				    "upper cut on number of hits on track"  ,
				    _highcutNHitsOnTrack ,
				    (int) 10. ) ;

	registerOptionalParameter( "lowCutPhi" , 
				    "lower cut on phi"  ,
				    _lowcutPhi ,
				    (float) -1. ) ;

	registerOptionalParameter( "highCutPhi" , 
				    "upper cut on phi"  ,
				    _highcutPhi ,
				    (float) 1. ) ;

	registerOptionalParameter( "lowCutTanLambda" , 
				    "lower cut on tan lambda"  ,
				    _lowcutTanLambda ,
				    (float) -1.7 ) ;

	registerOptionalParameter( "highCutTanLambda" , 
				    "upper cut on tan lambda"  ,
				    _highcutTanLambda ,
				    (float) 1.7 ) ;

	registerOptionalParameter( "lowCutdEdx" , 
				    "lower cut on dEdx"  ,
				    _lowcutdEdx ,
				    (float) 0. ) ;

	registerOptionalParameter( "highCutdEdx" , 
				    "upper cut on dEdx"  ,
				    _highcutdEdx ,
				    (float) 100000. ) ;

	registerOptionalParameter( "lowCutCharge" , 
				    "lower cut on charge of cluster"  ,
				    _lowcutCharge ,
				    (float) 0. ) ;

	registerOptionalParameter( "highCutCharge" , 
				    "upper cut on charge of cluster"  ,
				    _highcutCharge ,
				    (float) 100000. ) ;

	registerOptionalParameter( "lowCutD0" , 
				    "lower cut on D0"  ,
				    _lowcutD0 ,
				    (float) 0. ) ;

	registerOptionalParameter( "highCutD0" , 
				    "upper cut on D0"  ,
				    _highcutD0 ,
				    (float) 0. ) ;

	registerOptionalParameter( "lowCutZ0" , 
				    "lower cut on Z0"  ,
				    _lowcutZ0 ,
				    (float) 0. ) ;

	registerOptionalParameter( "highCutZ0" , 
				    "upper cut on Z0"  ,
				    _highcutZ0 ,
				    (float) 0. ) ;

	registerOptionalParameter( "lowCutOmega" , 
				    "lower cut on Omega"  ,
				    _lowcutOmega ,
				    (float) 0. ) ;

	registerOptionalParameter( "highCutOmega" , 
				    "upper cut on Omega"  ,
				    _highcutOmega ,
				    (float) 0. ) ;

	registerOptionalParameter( "lowCutX" , 
				    "lower cut on x position of hit"  ,
				    _lowcutX ,
				    (double) 0. ) ;

	registerOptionalParameter( "highCutX" , 
				    "upper cut on x position of hit"  ,
				    _highcutX ,
				    (double) 14.08 ) ;

	registerOptionalParameter( "lowCutY" , 
				    "lower cut on y position of hit"  ,
				    _lowcutY ,
				    (double) 0. ) ;

	registerOptionalParameter( "highCutY" , 
				    "upper cut on y position of hit"  ,
				    _highcutY ,
				    (double) 14.08 ) ;
				    
	registerOptionalParameter( "lowCutXTrack" , 
				    "lower cut on x position of track"  ,
				    _lowCutXTrack ,
				    (double) 0. ) ;

	registerOptionalParameter( "highCutXTrack" , 
				    "upper cut on x position of track"  ,
				    _highCutXTrack ,
				    (double) 14.08 ) ;

	registerOptionalParameter( "lowCutYTrack" , 
				    "lower cut on y position of track"  ,
				    _lowCutYTrack ,
				    (double) 0. ) ;

	registerOptionalParameter( "highCutYTrack" , 
				    "upper cut on y position of track"  ,
				    _highCutYTrack ,
				    (double) 14.08 ) ;

	registerOptionalParameter( "lowCutZ" , 
				    "lower cut on z position of hit"  ,
				    _lowcutZ ,
				    (double) 0. ) ;

	registerOptionalParameter( "highCutZ" , 
				    "upper cut on z position of hit"  ,
				    _highcutZ ,
				    (double) 270. ) ;
				    
	registerOptionalParameter( "lowCutTrackLength" , 
				    "lower cut track length"  ,
				    _lowCutTrackLength ,
				    (double) 5. ) ;
				    
	registerOptionalParameter( "highCutTrackLength" , 
				    "high cut track length"  ,
				    _highCutTrackLength ,
				    (double) 100. ) ;	    

	registerProcessorParameter( "CutIfFitFailed" , 
				    "cut if fit failed for a track (default: false)"  ,
				    _cutIfFitFailed ,
				    false ) ;
	
}
    
    
void CutApplicationProcessor::init() 
{ 
  // usually a good idea to
  printParameters() ;
}
    

void CutApplicationProcessor::processRunHeader( LCRunHeader* run) 
{ 
  run->parameters().setValue( _processorName + "_revision", "$Rev: 3387 $");

  for ( ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++ ) 
  {    
    if ( ! i->second->isOptional() || i->second->valueSet() ) 
    {
      run->parameters().setValue( _processorName + "_" + i->second->name(), 
        i->second->value() );
     }
  }
} 

    
void CutApplicationProcessor::processEvent( LCEvent * evt ) 
{ 
  //read input data
  streamlog_out(MESSAGE3) << "Event: " << evt->getEventNumber() << std::endl;

  LCCollectionVec* inputCol = 0; 
  try
  {
    inputCol = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
  } //end of try {
  catch(DataNotAvailableException)
  {
    // if there is not input collection give a message to the user
    streamlog_out(MESSAGE2)<<" processEvent:" <<" input collection "<<_inputColName
      <<" cannot be found in event "<<evt->getEventNumber()<<std::endl;
    return;
  }
    
  typedef std::vector< std::pair<TrackerHit*, double> > HitDouble;
   // first perform the event specific cuts
   if ( parameterSet("lowCutNTracks") && inputCol->getNumberOfElements() < _lowcutNTracks) return;
   if ( parameterSet("highCutNTracks") && inputCol->getNumberOfElements() > _highcutNTracks) return;
    
    
   // create the output collection with a string defining the TYPE (see lcio.h)
   LCCollectionVec* outputCol = new LCCollectionVec( LCIO::TRACK ); 
   // set the transient flag as definded by the processor parameter
   outputCol->setTransient( (_outputIsTransient != 0) );
   // the output collection is a subset of the input collection
   // only the pointers are stored, the track is not copied
   outputCol->setSubset();
    
   for ( LCCollectionVec::iterator trackIter = inputCol->begin(); trackIter < inputCol->end(); trackIter++) 
   {
     streamlog_out(DEBUG1)<<"looping tracks"<<std::endl;

     Track* track = dynamic_cast<Track*>( *trackIter ) ;
	
     //cast returns 0-pointer if cast fails
     if ( track == 0 ) continue; 
	
     // now check all cuts and continue without saving the track if cut is not fulfilled
	
     if ( parameterSet("lowCutNHitsOnTrack") &&  track->getTrackerHits().size() <
        static_cast<size_t>(_lowcutNHitsOnTrack) ) continue;
     if ( parameterSet("highCutNHitsOnTrack") &&  track->getTrackerHits().size() > 
       static_cast<size_t>(_highcutNHitsOnTrack) ) continue;
	
     if ( parameterSet("lowCutPhi") &&  track->getPhi() < _lowcutPhi ) continue;
     if ( parameterSet("highCutPhi") &&  track->getPhi() > _highcutPhi ) continue;
	
     if ( parameterSet("lowCutTanLambda") &&  track->getTanLambda() < _lowcutTanLambda ) continue;
     if ( parameterSet("highCutTanLambda") &&  track->getTanLambda() > _highcutTanLambda ) continue;
	
     if ( parameterSet("lowCutD0") &&  track->getD0() < _lowcutD0 ) continue;
     if ( parameterSet("highCutD0") &&  track->getD0() > _highcutD0 ) continue;
	
     if ( parameterSet("lowCutZ0") &&  track->getZ0() < _lowcutZ0 ) continue;
     if ( parameterSet("highCutZ0") &&  track->getZ0() > _highcutZ0 ) continue;
	
     if ( parameterSet("lowCutOmega") &&  track->getOmega() < _lowcutOmega ) continue;
     if ( parameterSet("highCutOmega") &&  track->getOmega() > _highcutOmega ) continue;
	    
     if ( parameterSet("lowCutdEdx") &&  track->getdEdx() < _lowcutdEdx ) continue;
     if ( parameterSet("highCutdEdx") &&  track->getdEdx() > _highcutdEdx ) continue;

     //double chipHeight = 14.08;

     if ( parameterSet("highCutYTrack") &&  ( _chipHeight*tan(fabs(track->getPhi()))+
       track->getD0()/cos(fabs(track->getPhi())) ) > _highCutYTrack  ) continue;
     if ( parameterSet("highCutYTrack") &&  ( track->getD0()/cos(fabs(track->getPhi())) ) 
       > _highCutYTrack  ) continue;	
     if ( parameterSet("lowCutYTrack") &&   ( _chipHeight*tan(fabs(track->getPhi()))+
       track->getD0()/cos(fabs(track->getPhi())) ) <  _lowCutYTrack ) continue;
     if ( parameterSet("lowCutYTrack") &&   ( track->getD0()/cos(fabs(track->getPhi())) ) 
       < _lowCutYTrack ) continue;
	
     //double chipLength = 14.08;

     if ( parameterSet("highCutXTrack") &&  ( track->getD0()/sin(fabs(track->getPhi())) - 
       _chipLength/tan(fabs(track->getPhi())) ) > _highCutXTrack  ) continue;
     if ( parameterSet("highCutXTrack") &&  ( track->getD0()/sin(fabs(track->getPhi())) ) > 
        _highCutXTrack  ) continue;	
     if ( parameterSet("lowCutXTrack") &&   ( track->getD0()/sin(fabs(track->getPhi())) - 
       _chipLength/tan(fabs(track->getPhi())) ) <  _lowCutXTrack ) continue;
     if ( parameterSet("lowCutXTrack") &&   ( track->getD0()/sin(fabs(track->getPhi())) ) < 
       _lowCutXTrack ) continue;
		
     if ( _cutIfFitFailed && LCFlagImpl(track->getType()).bitSet( TrackFitterBase::FITFAILEDBIT ) ) continue;

     streamlog_out(DEBUG1)<<"looping hits"<<std::endl;

     TrackerHitVec hitVec = track->getTrackerHits();
     bool trackIsInvalid=false; // flag to know if cut has not been fulfilled
     float sumZ=0;
//	    cout << "nr of hits " << hitVector.size()<< endl;
     HitDouble HitPairVec;
//	    int hitNr=0;
	  
     // first calculate some track parameters from track:
     double phi =  track->getPhi(); // just to improve readability

    // unit vector from reference point to point of closest approach (pca) (xy plane only)
    CLHEP::Hep3Vector n_pca( -sin( phi ), cos ( phi ) );

    // reference point in xy projection
    CLHEP::Hep3Vector reference_point( track->getReferencePoint ()[0], track->getReferencePoint ()[1],
      track->getReferencePoint ()[2]);

    // point of closest approach (3D point which is closest in the xy plane)
    CLHEP::Hep3Vector pca =  reference_point  
      +  track->getD0() *  n_pca  // the xy part
      +  track->getZ0() * CLHEP::Hep3Vector(0,0,1); // the z part
	    
    streamlog_out(DEBUG4) << "PCA is " << pca[0] << " , " << pca[1] << " , " << pca[2] << std::endl;
	    
    // psi is the angle of the pca wrt the helix center
    // (Note: in the paper from Lasiuk et al. from 1998 this is Phi_0,
    //  while Psi is the angle in track direktion.
    //  As Phi is the angle in track direction in LCIO,
    //  Phi and psi are exchanged wrt. the Lasiuk paper.)
    double psi = track->getPhi() + ( track->getOmega()<0 ? -1 : 1 ) * M_PI_2;
	    
    // wrap psi to -2pi .. 2pi to prevent the while loop running for ages 
    // in case there are nonsense values for the track's phi
    if ( std::fabs(psi) > 20 ) 
    {
      streamlog_out(WARNING3) << "Psi is " << psi << ", check phi of the input tracks!"
        << std::endl;
    }

    psi = std::fmod(psi, 2*M_PI);

    for ( TrackerHitVec::iterator hitIter = hitVec.begin(); hitIter < hitVec.end(); hitIter++ ) 
    {
       const double *pos = (*hitIter)->getPosition();
   
       streamlog_out(DEBUG1)<<"D"<<std::endl;
			    
       if ( parameterSet("lowCutX") && pos[0] < _lowcutX ) trackIsInvalid=true;
       if ( parameterSet("highCutX") &&  pos[0] > _highcutX) trackIsInvalid=true;
	    
       if ( parameterSet("lowCutY") && pos[1] < _lowcutY ) trackIsInvalid=true;
       if ( parameterSet("highCutY") &&  pos[1] > _highcutY) trackIsInvalid=true;

       if ( parameterSet("lowCutZ") && pos[2] < _lowcutZ ) trackIsInvalid=true;
       if ( parameterSet("highCutZ") &&  pos[2] > _highcutZ) trackIsInvalid=true;
	    
       if ( parameterSet("lowCutCharge") && (*hitIter)->getdEdx() < _lowcutCharge ) 
         trackIsInvalid=true;
       if ( parameterSet("highCutCharge") && (*hitIter)->getdEdx() > _highcutCharge   )
	 trackIsInvalid=true;
	    
       TrackerHit* trackerHit = *hitIter;
       // A Hep3Vector for the test hit
       CLHEP::Hep3Vector hit(trackerHit->getPosition()[0],trackerHit->getPosition()[1],
         trackerHit->getPosition()[2] );
	      
       sumZ+=hit[2];
       double s; // the track length in the xy-plane (projection)
       if (  track->getOmega() != 0. ) 
       {
         double co = std::fabs(track->getOmega())*(hit[0] - pca[0]) + cos(psi);
	 double si = std::fabs(track->getOmega())*(hit[1] - pca[1]) + sin(psi);
	 streamlog_out(DEBUG2)<< "TrackFitterBase: co "<<co <<" : si " << si 
	   << " : psi " << psi << std::endl;
		
	 double phi =  ( co == 0 ? ( si < 0 ? - M_PI_2    : M_PI_2 ) : ( co > 0 ? - atan(si/co) : 
           M_PI - atan(si/co) ) );
		
	 // wrap phi to be in +- pi wrt. -psi (minus because the pca is at -psi wrt helix centre)
	 while ( -psi - phi >  M_PI ) phi+=2*M_PI ;
	 while ( -psi - phi < -M_PI ) phi-=2*M_PI ;
		
	 s = (-psi - phi) / track->getOmega();
	 //	  s = ( atan(si/co) + ( co < 0 ? M_PI : 0 ) - psi ) / track->getOmega();
		
	 //	  //This is the method from Lasiuk et al. It does not work because 
	 //	  // the atan only works for +- pi/2, but we need a full circle
	 //	  s = atan( ( (hit[1] - pca[1] ) * sin(phi) + (hit[0] -pca[0]) * cos(phi) ) /
	 //		    ( 1/std::fabs( track->getOmega() ) +
	 //		      (hit[0] - pca[0]) * sin(phi) -
	 //		      (hit[1] - pca[1]) * cos(phi)                          
	 //		    )
	 //		  )
	 //	      /  track->getOmega() ;
	 streamlog_out(DEBUG3)<< "TrackFitterBase: Hit "<<hit[0] <<" : " <<hit[1] <<" : "<<hit[2] 
	   <<" ; s is " << s << std::endl;
       }
       else
       {
         s =  (hit[1] - pca[1] ) * sin(phi) + (hit[0] -pca[0]) * cos(phi);	// tracklength in xy projection
		
	 streamlog_out(DEBUG3)<< "TrackFitterBase: Hit "<<hit[0] <<" : " <<hit[1] <<" : "<<hit[2] 
	   <<" ; s is " << s << std::endl;
       }

       s = s * std::sqrt( 1 + track->getTanLambda()*track->getTanLambda()); // here comes the third dimension
       std::pair<TrackerHit*, double> HitPair( trackerHit , s);
       HitPairVec.push_back( HitPair );
     }// for hitIter
     
     sort( HitPairVec.begin(), HitPairVec.end(), compareHit );
	
     //effective length of track is distance between outer hits times n/(n-1):
     //distance between outer points on track
     double l = fabs( (*HitPairVec.begin()).second - (*(HitPairVec.end()-1)).second );
     //effective length
     double l_eff = l*hitVec.size()/(hitVec.size()-1);
     if (parameterSet("lowCutTrackLength") &&  l_eff < _lowCutTrackLength) trackIsInvalid=true;
     if (parameterSet("highCutTrackLength") &&  l_eff > _highCutTrackLength) trackIsInvalid=true;
     if (!trackIsInvalid) outputCol->addElement( *trackIter );
   }// for trackIter
    
   // only add non-empty collections to the event
   if ( outputCol->empty() ) delete outputCol;
   else  evt->addCollection(outputCol, _outputColName);
    
}// process event
}// namecpace marlintpc
