 #include "HitAndTrackChargeProcessor.h"

//C++
#include <iostream>
#include <cmath>
#include <sstream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/Track.h>
#include <EVENT/TrackerHit.h>

// Aida
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
// endif

#include <marlin/Global.h>

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

HitAndTrackChargeProcessor aHitAndTrackChargeProcessor;
    
    HitAndTrackChargeProcessor::HitAndTrackChargeProcessor(): Processor("HitAndTrackChargeProcessor")
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACK,
				"InputTracks" , 
				"Name of the input tracks collection (default: TPCTracks)"  ,
				_tracksColName ,
				std::string("TPCTracks") ) ;
    registerInputCollection(    LCIO::TRACKERHIT,
				"InputHits" , 
				"Name of the input hits collection (default: TPCHits)"  ,
				_hitsColName ,
				std::string("TPCHits") ) ;
    // modify processor description
    _description = "Count hits in events and tracks, and track per event" ;

  registerProcessorParameter( "MaxChargePerHit" , 
			      "Maximum value in ChargePerHit histograms (default: ?? )"  ,
			       _maxChargePerHit,
			      float(10000) );

  registerProcessorParameter( "MaxChargePerTrack" , 
			      "MaxChargePerTrack Maximum value in ChargePerHit histogram (default: ?? )"  ,
			       _maxChargePerTrack,
			      float(100000) );

  registerProcessorParameter( "Max_dEdx" , 
			      "Max_dEdx  Maximum value in dEdx histogram (default: ?? )"  ,
			       _max_dEdx,
			      float(1000) );

  registerProcessorParameter( "MaxZ" , 
			      "MaxZ Maximum z value of highest bin in case of z binning (default: 260)",
			       _maxZ,
			      float(260) );
  registerProcessorParameter( "MinZ" , 
			      "MinZ Minimum z value of lowest bin in case of z binning (default: 0)",
			       _minZ,
			      float(0) );
  registerProcessorParameter( "nZBins" , 
			      "Number ob bins in z, 0 means one Histo without z cut (default: 10)",
			       _nZBins,
			      int(10) );
   
//  if (_nZBins>0)
//      _zBinWidth = (_maxZ - _minZ)/_nZBins;

  
//  _chargePerTrackHistos.resize(_nZBins +1);
//  _dEdxHistos.resize(_nZBins +1);
//  _chargePerHitHistos.resize(_nZBins +1);
//  _chargePerHitOnTrackHistos.resize(_nZBins +1);
}

HitAndTrackChargeProcessor::~HitAndTrackChargeProcessor()
{
}

void HitAndTrackChargeProcessor::init()
{ 
	// usually a good idea to
	printParameters() ;
	
	if (_nZBins>0)
	    _zBinWidth = (_maxZ - _minZ)/_nZBins;


	_chargePerTrackHistos.resize(_nZBins +1);
	_dEdxHistos.resize(_nZBins +1);
	_chargePerHitHistos.resize(_nZBins +1);
	_chargePerHitOnTrackHistos.resize(_nZBins +1);

	// check parameter:
	// _nZBins >=0
	if (_nZBins < 0)
	{
	    streamlog_out(WARNING3) << "invalid _nZBins < 0 ; setting to 0" 
				    << std::endl;
	    _nZBins = 0;
	}
	// _minZ < _maxZ

	// get a histograms from the AIDAProcessor
	
	// first the histos in z slices
	for (int i=0; i<=_nZBins ; i++)
	{
	    std::stringstream zRangeString;
	    std::stringstream zIndexString;
	    if (i < _nZBins)
		// no extention for the histo without z cut
	    {
		zRangeString << " "<<_minZ + (_maxZ - _minZ)*float(i)/float(_nZBins)
			     << " < z < "
			     << _minZ + (_maxZ - _minZ)*float(i+1)/float(_nZBins);
		zIndexString << i ;
	    }

	    string histoTitle = string("Charge per Hit") + zRangeString.str();
	    string histoName  = string("ChargePerHit") + zIndexString.str();

	    streamlog_out(DEBUG0) << histoName << " : " << histoTitle << endl;
	    _chargePerHitHistos[i] =   AIDAProcessor::histogramFactory(this)->
		createHistogram1D( histoName, 
				   histoTitle, 
				   100, 0, _maxChargePerHit);

	    histoTitle = string("Charge per Hit on Track") + zRangeString.str();
	    histoName  = string("ChargePerHitOnTrack") + zIndexString.str();

	    streamlog_out(DEBUG0) << histoName << " : " << histoTitle << endl;
	    _chargePerHitOnTrackHistos[i] =   AIDAProcessor::histogramFactory(this)->
		createHistogram1D( histoName , 
				   histoTitle , 
				   100, 0, _maxChargePerHit);

	    streamlog_out(DEBUG0) << histoName << " : " << histoTitle << endl;
	    histoTitle = string("Charge per Track") + zRangeString.str();
	    histoName  = string("ChargePerTrack") + zIndexString.str();

	    _chargePerTrackHistos[i] =   AIDAProcessor::histogramFactory(this)->
		createHistogram1D( histoName, 
				   histoTitle, 
				   100, 0, _maxChargePerTrack);

	    histoTitle = string("Charge per Track Length") + zRangeString.str();
	    histoName  = string("dEdx") + zIndexString.str();

	    streamlog_out(DEBUG0) << histoName << " : " << histoTitle << endl;
	    _dEdxHistos[i] =   AIDAProcessor::histogramFactory(this)->
		createHistogram1D( histoName, 
				   histoTitle , 
				   100, 0, _max_dEdx);
	}// for i

	streamlog_out(DEBUG0)<<"_chargePerHitHistos.size() + "<<_chargePerHitHistos.size()<<std::endl;
	streamlog_out(DEBUG0)<<"_chargePerHitOnTrackHistos.size() = "<<_chargePerHitOnTrackHistos.size()<<std::endl;
	streamlog_out(DEBUG0)<<"_chargePerTrackHistos.size() + "<<_chargePerTrackHistos.size()<<std::endl;
	streamlog_out(DEBUG0)<<"_dEdxHistos.size() = "<<_dEdxHistos.size()<<std::endl;
	
}

void HitAndTrackChargeProcessor::processRunHeader( EVENT::LCRunHeader* run )
{
}
    
void HitAndTrackChargeProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data
    streamlog_out(MESSAGE3) << "Event: " << evt->getEventNumber() << std::endl;

    // we need a LCCollectionVec to use iterators, as LCCollectio is not a std::vector
    LCCollectionVec* tracksCollection=0;
    try{
	tracksCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _tracksColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// there are no tracks
	// nevermind, just catch the exception and continue 
    }


    // if there are tracks count them and the hits per track
    if( tracksCollection != 0 )
    {
	for(LCCollectionVec::iterator it = tracksCollection->begin() ; 
	    it < tracksCollection->end(); it ++ )
	{
	    Track * track = dynamic_cast<Track *>(*it) ;
	

	    // loop the hits and fill the charge into histo 
	    // also calculate charge on track and fill it into the specific histo
	    float chargeOnTrack=0;
	    
	    streamlog_out(DEBUG3) << "track has collection with " 
				  <<track->getTrackerHits().size() 
				  <<" hits"<< std::endl;
	    
	    float sumZ=0;
	    for( TrackerHitVec::const_iterator hitIter = track->getTrackerHits().begin() ; 
		 hitIter < track->getTrackerHits().end(); hitIter ++ )
	    {
		streamlog_out(DEBUG2) << "charge in hit is  " 
				  <<(*hitIter)->getdEdx() << std::endl;

		chargeOnTrack+=(*hitIter)->getdEdx() ;
		
		
		int zBin = static_cast<int>( ((*hitIter)->getPosition()[2] - _minZ) 
					     / _zBinWidth );

		sumZ += (*hitIter)->getPosition()[2];
		if ( (zBin >= 0) && (zBin < _nZBins) )
		    _chargePerHitOnTrackHistos[zBin]->fill((*hitIter)->getEDep () );

		_chargePerHitOnTrackHistos[_nZBins]->fill((*hitIter)->getEDep () );
	    }//for hitIter

	    if (_nZBins > 0)
	    {
		// zBin of the track (from mean z if the track)
		int zBin = static_cast<int>( sumZ / track->getTrackerHits().size() 
					     / _zBinWidth );
		
		if ( (zBin >= 0) && (zBin < _nZBins) )
		{
		    _chargePerTrackHistos[zBin]->fill(chargeOnTrack);
		    _dEdxHistos[zBin]->fill( track->getdEdx() );
		} 
	    } // in _nZBins >0

	    _chargePerTrackHistos[_nZBins]->fill(chargeOnTrack);

	    streamlog_out(DEBUG3) << "track dEdx is"
				  << track->getdEdx() << std::endl;
	    
	    _dEdxHistos[_nZBins]->fill( track->getdEdx() );
	}
		
    }

    LCCollectionVec* hitsCollection=0;
    try{
	 hitsCollection = dynamic_cast<LCCollectionVec*>( evt->getCollection( _hitsColName ) ) ;
    }
    catch(DataNotAvailableException &e)
    {
	// there are no hits
	// continue with next event
	return;
    }

    // iterate the hits to fill the charge into histograms
    if( hitsCollection != 0 )
    {
	for( LCCollectionVec::iterator hitIter = hitsCollection->begin() ; 
	    hitIter < hitsCollection->end(); hitIter ++ )
	{
	    TrackerHit *hit = dynamic_cast<TrackerHit *>(*hitIter) ;
	
	    // a hit is point like , it's not dEdx but charge
	    int zBin = static_cast<int>( (hit->getPosition()[2] - _minZ)
					 / _zBinWidth ); 
	    if ( (zBin >= 0) && (zBin < _nZBins) )
		_chargePerHitHistos[zBin]->fill(hit->getEDep () );

	    _chargePerHitHistos[_nZBins]->fill(hit->getEDep () );
	}
    }
    
}


}// namespace 
