#include "TimePixEventDisplayCreator.h"


namespace marlintpc{
  TimePixEventDisplayCreator::TimePixEventDisplayCreator(double inXMin, double inXMax, double inYMin, double inYMax, double inZMin, double inZMax, int InXPixel, int InYPixel)  // constructor
  { _xMin = inXMin;
    _xMax = inXMax;
    _yMin = inYMin;
    _yMax = inYMax;
    _zMin = inZMin;
    _zMax = inZMax;
    _xPixel = InXPixel;
    _yPixel = InYPixel;
    
    
  }
  
    TimePixEventDisplayCreator::TimePixEventDisplayCreator()  // default constructor
  { _xMin = -0.00275;
    _xMax = 14.07725;
    _yMin = -0.00275;
    _yMax = 14.07725;
    _zMin = 0.0;
    _zMax = 265;
    _xPixel = 256;
    _yPixel = 256;
    
    
  }
  
  TimePixEventDisplayCreator::~TimePixEventDisplayCreator()  // destructor
  {
    
  }  
  
  TH2F *TimePixEventDisplayCreator::GetTH2F_TracksXY(IMPL::LCCollectionVec* inputColVec, unsigned int EventNr, unsigned int RunNr, unsigned int showcharge){
        
    if( !inputColVec->empty() ){	// one can only create a histo if one has data...
        
    // build histo name
    std::stringstream eventNumberString;
    eventNumberString << EventNr;
    std::stringstream runNumberString;
    runNumberString << RunNr;
    std::string histoName;
    if (showcharge != 1)
      histoName = "Run"+runNumberString.str()+"Event"+eventNumberString.str()+"XYPlane";
    else
      histoName = "Run"+runNumberString.str()+"Event"+eventNumberString.str()+"ChargeXYPlane";
    
    // create histo
    TH2F* myHisto = new TH2F(histoName.c_str(), histoName.c_str(), _xPixel, _xMin, _xMax, _yPixel, _yMin, _yMax);
    
    double pos[3] = {0,0,0};
    int trackNumber = 0;
    
    for( IMPL::LCCollectionVec::iterator inputObjectIter = inputColVec->begin(); inputObjectIter < inputColVec->end(); inputObjectIter++ ){
     
     EVENT::Track * track;
     try								
      {
	track = dynamic_cast<EVENT::Track *>(*inputObjectIter) ;	// dynamic cast the input object (LCObject) to what it really is e.g.
      }
      catch (DataNotAvailableException &e)			
      {
	return (0);							// no input data, just return
      }
 
      // now we have hands on a a track of our event - lets work on this 
      EVENT::TrackerHitVec hitVector = track->getTrackerHits();
      trackNumber++;
      for (EVENT::TrackerHitVec::iterator iterHits = hitVector.begin(); iterHits <  hitVector.end(); iterHits ++){
	
	EVENT::TrackerHit* hit = dynamic_cast<TrackerHit*>(*iterHits);
	const EVENT::LCObjectVec* rawHits = & hit->getRawHits();
	
	for(EVENT::LCObjectVec::const_iterator iterData = rawHits->begin(); iterData < rawHits->end(); iterData++){
	  
	  EVENT::TrackerData * data = dynamic_cast<TrackerData *>(*iterData);
	  FloatVec adcValues = data->getChargeValues();
	  
	  for(unsigned int iADCvalues = 0; iADCvalues < adcValues.size(); iADCvalues++){
	    int cellID0 = data->getCellID0()+ iADCvalues;					// looping pixels in rawhit vec
	    pos[0] = ( _xPixel - 1 - (cellID0 % _xPixel) ) * ( _xMax - _xMin ) / _xPixel;
	    pos[1] = ( cellID0 / _yPixel ) * ( _yMax - _yMin ) / _yPixel ;
	    
	    // finaly fill histo
	    if (showcharge != 1)
	      myHisto->Fill(pos[0],pos[1],trackNumber);
	    else
	      myHisto->Fill(pos[0],pos[1], adcValues.at(iADCvalues) );
	    
	  }
	  
	} // end for loop rawHits

      }   // end for loop tracker hits
           
      // display fitted track aswell
      std::stringstream trackNameString;
      trackNameString<< trackNumber;
      std::string trackName = "TrackNumber"+trackNameString.str()+"XYPlane";
      TF1 *xyTrack= new TF1(trackName.c_str(),"[0]*x+[1]",_xMin+0.1,_xMax-0.1);
      double c=tan(track->getPhi());
      double d=track->getD0()/cos(track->getPhi());
      xyTrack->FixParameter(0,c);
      xyTrack->FixParameter(1,d);
      myHisto->GetListOfFunctions()->Add(xyTrack);  
        
    } // end for loop over inputCollection

    // setup axis
    myHisto->GetXaxis()->SetTitle("x [mm]");
    myHisto->GetYaxis()->SetTitle("y [mm]");
    myHisto->SetTitle("");

    return(myHisto);    
    
    }  // end if( !inputColVec->empty() )
    else
      return (0);
    
  } // end GetTH2F_XY
  
  
  TH2F *TimePixEventDisplayCreator::GetTH2F_TracksZX(IMPL::LCCollectionVec* inputColVec, unsigned int EventNr, unsigned int RunNr){
     
    if( !inputColVec->empty() ){	// one can only create a histo if one has data...
        
    // build histo name
    std::stringstream eventNumberString;
    eventNumberString << EventNr;
    std::stringstream runNumberString;
    runNumberString << RunNr;
    std::string histoName;
    histoName = "Run"+runNumberString.str()+"Event"+eventNumberString.str()+"ZXPlane";
    
    // create histo
    TH2F* myHisto = new TH2F(histoName.c_str(), histoName.c_str(),250, _zMin, _zMax, _xPixel/2.0, _xMin, _xMax);
    
    int trackNumber = 0;
    
    for( IMPL::LCCollectionVec::iterator inputObjectIter = inputColVec->begin(); inputObjectIter < inputColVec->end(); inputObjectIter++ ){	// loop over tracks in input col
     
     EVENT::Track * track;
     try								
      {
	track = dynamic_cast<EVENT::Track *>(*inputObjectIter) ;	// dynamic cast the input object (LCObject) to what it really is e.g.
      }
      catch (DataNotAvailableException &e)			
      {
	return (0);							// no input data, just return
      }
 
      // now we have hands on a a track of our event - lets work on this 
      EVENT::TrackerHitVec hitVector = track->getTrackerHits();
      trackNumber++;
      for (EVENT::TrackerHitVec::iterator iterHits = hitVector.begin(); iterHits <  hitVector.end(); iterHits ++){
	
	EVENT::TrackerHit* hit = dynamic_cast<TrackerHit*>(*iterHits);

	myHisto->Fill(hit->getPosition()[2],hit->getPosition()[0],trackNumber);

      }   // end for loop tracker hits
           
      // display fitted track aswell
      std::stringstream trackNameString;
      trackNameString<< trackNumber;
      std::string trackName = "TrackNumber"+trackNameString.str()+"ZXPlane";
      TF1 *zxTrack= new TF1(trackName.c_str(),"[0]*x+[1]",_zMin+0.1,_zMax-0.1);
      double a = 1 / ( ( track->getTanLambda() ) * sqrt( 1 + tan ( track->getPhi())*tan(track->getPhi() ) ) );
      double b = - ( track->getZ0() ) * a - ( (track->getD0() ) * sin( track->getPhi() ) );
      zxTrack->FixParameter(0,a);
      zxTrack->FixParameter(1,b);
      myHisto->GetListOfFunctions()->Add(zxTrack);
               
    } // end for loop over inputCollection

    // setup axis
    myHisto->GetXaxis()->SetTitle("z [mm]");
    myHisto->GetYaxis()->SetTitle("x [mm]");
    myHisto->SetTitle("");

    return(myHisto);    
    
    }  // end if( !inputColVec->empty() )
    else
      return (0);
    
    
    
    
  } // end GetTH2F_TracksZX
  
  
  TH2F *TimePixEventDisplayCreator::GetTH2F_TracksZY(IMPL::LCCollectionVec* inputColVec, unsigned int EventNr, unsigned int RunNr){
     
    if( !inputColVec->empty() ){	// one can only create a histo if one has data...
        
    // build histo name
    std::stringstream eventNumberString;
    eventNumberString << EventNr;
    std::stringstream runNumberString;
    runNumberString << RunNr;
    std::string histoName;
    histoName = "Run"+runNumberString.str()+"Event"+eventNumberString.str()+"ZYPlane";
    
    // create histo
    TH2F* myHisto = new TH2F(histoName.c_str(), histoName.c_str(),250, _zMin, _zMax, _yPixel/2.0, _yMin, _yMax);
    
    int trackNumber = 0;
    
    for( IMPL::LCCollectionVec::iterator inputObjectIter = inputColVec->begin(); inputObjectIter < inputColVec->end(); inputObjectIter++ ){	// loop over tracks in input col
     
     EVENT::Track * track;
     try								
      {
	track = dynamic_cast<EVENT::Track *>(*inputObjectIter) ;	// dynamic cast the input object (LCObject) to what it really is e.g.
      }
      catch (DataNotAvailableException &e)			
      {
	return (0);							// no input data, just return
      }
 
      // now we have hands on a a track of our event - lets work on this 
      EVENT::TrackerHitVec hitVector = track->getTrackerHits();
      trackNumber++;
      for (EVENT::TrackerHitVec::iterator iterHits = hitVector.begin(); iterHits <  hitVector.end(); iterHits ++){
	
	EVENT::TrackerHit* hit = dynamic_cast<TrackerHit*>(*iterHits);

	myHisto->Fill(hit->getPosition()[2],hit->getPosition()[1],trackNumber);

      }   // end for loop tracker hits
           
      // display fitted track aswell
      std::stringstream trackNameString;
      trackNameString<< trackNumber;
      std::string trackName = "TrackNumber"+trackNameString.str()+"ZYPlane";
      TF1 *zyTrack= new TF1(trackName.c_str(),"[0]*x+[1]",_zMin+0.1,_zMax-0.1);		//FIXME not the right parameters up to now
      
      double e = tan(track->getPhi()) / ( ( track->getTanLambda() ) * sqrt( 1 + tan ( track->getPhi())*tan(track->getPhi() ) ) );
      double f = - ( ( track->getZ0() *tan(track->getPhi()) ) / ( ( track->getTanLambda() ) * sqrt( 1 + tan ( track->getPhi())*tan(track->getPhi() ) ) ) ) 
		 - ( (track->getD0() ) * sin( track->getPhi() ) * tan(track->getPhi() )) + (track->getD0()/cos(track->getPhi() ) ); 

      zyTrack->FixParameter(0,e);
      zyTrack->FixParameter(1,f); 
      myHisto->GetListOfFunctions()->Add(zyTrack);
      
               
    } // end for loop over inputCollection

    // setup axis
    myHisto->GetXaxis()->SetTitle("z [mm]");
    myHisto->GetYaxis()->SetTitle("y [mm]");
    myHisto->SetTitle("");

    return(myHisto);    
    
    }  // end if( !inputColVec->empty() )
    else
      return (0);
    
    
    
    
  } // end GetTH2F_TracksZX
  
  
} // end namespace marlintpc
