#ifndef Processor_h
#define InhEFieldCalculationProcessor_h 1


//LCIO
#include "lcio.h"

//MARLIN
#include "marlin/Processor.h"

//C++
#include "string"

//GEAR
#include "gearimpl/RectangularPadRowLayout.h"
#include "gearimpl/FixedPadSizeDiskLayout.h"

//MARLINTPC
#include "EFieldVector.h"

namespace marlintpc
{

  class InhEFieldCalculationProcessor : public marlin::Processor
  {

    public:

      virtual Processor*  newProcessor()
      {
        return new InhEFieldCalculationProcessor ;
      }


      InhEFieldCalculationProcessor() ;
      virtual ~InhEFieldCalculationProcessor() {}

      /** Called at the begin of the job before anything is read.
       *  Use to initialize the processor, e.g. book histograms.
       */
      virtual void init() ;

      /** Called for every run.
       */
      virtual void processRunHeader(EVENT::LCRunHeader* run) ;

      /** Called for every event - the working horse.
       */
      virtual void processEvent(EVENT::LCEvent * evt) ;


      virtual void check(EVENT::LCEvent * evt) ;


      /** Called after data processing for clean-up.
       */
      virtual void end() ;


      marlintpc::EFieldVector* calculateEField(int x, int y, int z,  LCCollection* ions);


    protected:

      // Input/output collection names
      std::string _inputCollectionName;
      std::string _outputCollectionName;

      // Input parameters
      int _transient;
      int _xBins, _yBins, _zBins;

      double _xBinLength, _yBinLength, _zBinLength;

      gear::PadRowLayout2D const* _padLayout;
      gear::RectangularPadRowLayout const* _rectPL;
      gear::FixedPadSizeDiskLayout const* _circPL;

  };
}
#endif
