/*
 * MarlinTPCExceptions.hh
 *
 * Definition of the exception hierarchy for MarlinTPC
 *
 *  Created on: Jul 13, 2012
 *      Author: caiazza
 */

#ifndef MARLINTPCEXCEPTIONS_HH_
#define MARLINTPCEXCEPTIONS_HH_

#include <string>
#include <exception>

namespace marlintpc
{
/**
 * Base exception for MarlinTPC classes.
 * All other exceptions of this hierarchy should extend this
 * @author caiazza
 * @version Jul 13, 2012
 */
class Exception : public std::exception
{
protected:
    std::string message ;

    Exception(){  /*no_op*/ ; }

public:
    virtual ~Exception() throw() { /*no_op*/; }

    Exception( const std::string& text ):
            message(std::string("marlintpc::Exception: ") + text)
            { /*no_op*/ }

    Exception(const std::string& ClassName, const std::string& Message):
        message(std::string("marlintpc::Exception in object of class ") + ClassName + " : " + Message)
    { /*no_op*/; }

    virtual const char* what() const  throw()
            { return  message.c_str() ; }

};

/**
 * Base exception for error involving some data having an invalid value
 */
class InvalidDataException : public Exception
{
public:
    virtual ~InvalidDataException() throw() { /*no_op*/ }
    InvalidDataException():
        Exception("The data is not valid.")
    { /*no_op*/ }

    InvalidDataException(const std::string &err):
        Exception(err)
    { /*no_op*/ }

    InvalidDataException(const std::string& ClassName, const std::string& Message):
        Exception(ClassName, Message)
    { /*no_op*/ }
};


/**
 * Exception to be thrown in the specific case that the invalid data is a null pointer
 */
class NullPointerException : public InvalidDataException
{
public:
    NullPointerException(const std::string& ClassName, const std::string& Message ):
        InvalidDataException(ClassName, Message) {}
};

}


#endif /* MARLINTPCEXCEPTIONS_HH_ */
