#ifndef TIMEPIX_TOT_DISTRIBUTION_PROCESSOR_H
#define TIMEPIX_TOT_DISTRIBUTION_PROCESSOR_H 1

//LCIO
#include <lcio.h>

//MARLIN
#include "marlin/Processor.h"
#include "TimePixPixelMode.h"
#include "TimePixMapListener.h"

//root
#include "TROOT.h"
#include "TH2F.h"

namespace AIDA{
  class IHistogram1D;
  class IHistogram2D;
}


namespace marlintpc
{

    class TrackFitterFactory;
//    class TrackFitterBase;

    /** Creates XY Histo with mean Charge per Pixel
     *
     *  <h4>Input - Prerequisites</h4>
     *  \li A Collection of tracker data
     *
     *  <h4>Output</h4>
     *  \li Histograms of the TOT value distribution
     *
     *  @param InputTrackes Name of input Hits collection (default: TPCHits)
     *  @param HistogramMinValueTOT  Minimum value in TOT histogram (default: 0.)
     *  @param HistogramMaxValueTOT  Maximal value in TOT histogram (default: 1000.) 
     *
     *
     *  @author Simone Zimmermann, University of Bonn  -- Major Changes Applied C.Brezina
     */
    class TimePixTOTDistributionProcessor : public marlin::Processor{
			      
    public:
    
	virtual Processor*  newProcessor() { return new TimePixTOTDistributionProcessor ; }

	TimePixTOTDistributionProcessor();
    
	~TimePixTOTDistributionProcessor();

	virtual void init();

	virtual void processRunHeader( EVENT::LCRunHeader* run ) ;

	virtual void processEvent( LCEvent * evt ) ;
	
	virtual void end( ) ; 
    
    protected:

     std::string _inputColName ;    ///< the name of the input collection
     
     TH2F * _distributionXYHisto; ///< The histogram of the TOT values
     TH2F * _NrOfHitsXYHisto; ///< Number of ToT hits in the Pixel
     TH2F *_MeanChargePerPixelXY;
     TH1F *_MeanChargeHisto;	///< shows the histo of the entries in _MeanChargePerPixelXY
 
     std::string _selectedMode;///<- the mode that should be plottet
     int _plotMode; // int version of above parameter
     float  _histogramMinValueX; ///< Minimum value in TOT histogram
     float  _histogramMaxValueX; ///< Maximum value in TOT histogram
     float  _histogramMinValueY; ///< Minimum value in TOT histogram
     float  _histogramMaxValueY; ///< Maximum value in TOT histogram
     
     int _nBinsX;
     int _nBinsY;
     int _maxMeanCharge;
     
     tpcconddata::TimePixMap * _timePixMap;

    };

}

#endif // TOT_DISTRIBUTION_PROCESSOR_H
