#ifndef CutHitsOnTracksProcessor_h
#define CutHitsOnTracksProcessor_h 1

//marlin
#include "marlin/Processor.h"
#include "marlin/Global.h"
//#include "lcio.h"
//#include <string>
//#include <vector>
//#include <EVENT/TrackerHit.h>

// LCIO
#include "EVENT/LCIO.h"
#include "EVENT/LCCollection.h"
#include "EVENT/TrackerHit.h"
#include <IMPL/LCCollectionVec.h>
#include "IMPL/TrackImpl.h"
#include <IMPL/LCFlagImpl.h>

// Aida
#ifdef MARLIN_USE_AIDA
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#endif

//namespace AIDA {
//  class IHistogram1D;
//  class IHistogram2D;
//}


namespace marlintpc
{

/* the following comment is tranfered to doxygen documentation
  * which begins with a second asterix, or in case of on line comment an extra third slash
  */
/** 
 *  CutHitsOnTracksProcessor is a Cut application Processor to be applied on reconstructed hits of a track
 *  @author C:Brezina (University of Bonn)
 *
 *  <h4>Input - Prerequisites</h4>
 *  \li A collection of track candidates or seen tracks (lcio::Tracks) containing the TrackerHits.<br>
 *	Note: The LCIO::TRBIT_HITS flag has to be set when this collection is created so
 *      the hits are stored in the track.
 *
 *  <h4>Output</h4>
 *  \li Collection of straight tracks.
 *
 * @param InputTrackCandidates: The the name of the input collection of track candidates (default: CompleteTPCTrackCandidates)
 * @param lowXCut: minimal Position of Hits on X-Axis in mm
 * @param lowYCut: minimal Position of Hits in Y-Axis in mm
 * @param highXCut: highest Position of Hits on X-Axis in mm
 * @param highYCut: highest Position of Hits in Y-Axis in mm
 * @param minNrOfHitsOnTrack: If a Track has less than N Hits it is thrown away
 * @param OutputSeedTracks: The name of the collection the seed tracks be stored under (default: CutTPCSeedTracks)
 *  
 */
//@param SetOutputTransient If not 0 the output collection is set transient (default: 0)

class CutHitsOnTracksProcessor : public marlin::Processor
{
  
    public:
        
        /** Default constructor
	*/
	CutHitsOnTracksProcessor();


	/** Destructor
	*/
	~CutHitsOnTracksProcessor();


	/** Return a new instance of this processor
	*/
	Processor * newProcessor();


	/** Called at the begin of the job before anything is read.
	*/
	void init();


	/** Called once per run to process the event's header
	*/
	void processRunHeader (EVENT::LCRunHeader *run);


	/** Called for every event - the working horse.
	*/
	virtual void processEvent(EVENT::LCEvent *evt);


	/** Called for every event - right after processEvent() 
	 *has been called for all processors.
	*/
	virtual void check (EVENT::LCEvent *evt);


	/** Called after data processing for clean up in the 
	 *  inverse order of the init() method so that resources 
	 *  allocated in the first processor also will be available for 
	 *  all following processors.
	*/
	virtual void end();

private:

	/** the name of the input collection of track candidates
	*/
	std::string _input_track_candidates_collection_name;

	/** the name of the collection the seed tracks be stored under
	*/
	std::string _output_seed_tracks_collection_name;

	/** Give the status which is set to the transient flag of the output collection.
	 *  It is an int instead of a boolean, because the processor parameter 
	 *  cannot be a boolean.
	 *  Hits and tracks can be set separately.
	 */
//	int _outputIsTransient; //< if not 0 the hits collection is set transient 
	                        //(default 0)

	float _lowXCut;  ///<- minimal Position of Hits on X-Axis in mm
	float _lowYCut;  ///<- minimal Position of Hits in Y-Axis in mm
	float _highXCut;  ///<- highest Position of Hits on X-Axis in mm
	float _highYCut;  ///<- highest Position of Hits in Y-Axis in mm

	int _minNrOfHitsOnTrack; ///<-If a Track has less than N Hits it is thrown away

};

}// namespace marlintoc

#endif // CutHitsOnTracksProcessor



