#ifndef HELIKALTRACK_H
#define HELIKALTRACK_H

#include "AttKalTrack.h"

//KalTest
#include <kaltest/TKalTrack.h>

namespace marlintpc
{

/** This is the helix track class, which inherits from AttKalTrack and TKalTrack.
 *  Kalman filter algorithm comes from TKalTrack which is a class in KalTest package.
 *
 *  @author Bo Li, Tsinghua University
 *  @author Keisuke Fujii, KEK
 */

class HeliKalTrack : public AttKalTrack, public TKalTrack
{
    public:
        HeliKalTrack(const gear::TPCParameters* tp);

        HeliKalTrack(const HeliKalTrack& orig);

        /** Create a track by copying from original track
         */
        AttKalTrack* clone();

        /** Add a new site to track and update track parameters
         */
        bool doAddAndFilter(ModuleRowHit hit);

        /** Search track starting from hit
         */
        bool searchTrack(ModuleRowHit hit);

        /** Reset parameters and data of track
         */
        void reset();

        /** Get the Chi2 increment at one step
         */
        double getDeltaChi2();

        double getKappa();
        int    getNDF()
        {
            return TKalTrack::GetNDF();
        }
        double getChi2()
        {
            return TKalTrack::GetChi2();
        }

        /** Get state vector
         */
        TMatrixD getStateVec();

        /** Get track type: helix
         */
        inline int getType()
        {
            return 1;
        }

        /** Print track parameters
         */
        void printStateVector();


        /** Fill the track state (incl. cov matrix) at the beginning (4th hit) of the track into the given LCIO track.
        *  Smoothes back to the 4th hit and returns that state.
        */
        void toLCIOTrack(IMPL::TrackImpl* trk) ;

};


}
#endif  //HELIKALTRACK_H
