#ifndef HIT_IN_TRACKS_SPLITTER_PROCESSOR_H
#define HIT_IN_TRACKS_SPLITTER_PROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>
#include <set>

#include <EVENT/TrackerPulse.h>

namespace marlintpc
{
/**  The HitsInTracksSplitterProcessor splits hits with the same algorithm than the HitSpliterProcessor.
 *   However, it takes only those hits which are stored in tracks and keeps the association to
 *   the tracks (grouping).
 *
 *   @param InputCollectionName Name of the tracks with the hits to be split, default: TPCTracks
 *   @param SplitHitsCollectionName Name of the collection with split hits, default: OnlySplitHits
 *   @param PulsesInSplitHitsCollectionName Name of the collection with pulses contained in the split hits,
 *          default: PulsesInSplitHits
 *   @param OutputTracksCollectionName Name of the collection of tracks with split hits,
 *          default: TPCTracksWithSplitHits
 *   @param TPCConditonsCollectionName Name of the TPCConditons collection (for vDrift),
 *          default: TPCConditions
 *   @param VDriftOverride \c optional Use to overwrite the drift velocity from conditions data,
 *          or in case there is no conditions data (in mm / us )
 *
 * @author M. Killenberg, CERN
 *
 */

class HitsInTracksSplitterProcessor : public marlin::Processor
{
    public:

        virtual Processor*  newProcessor()
        {
            return new HitsInTracksSplitterProcessor;
        }

        HitsInTracksSplitterProcessor();

        virtual void init();

        virtual void processRunHeader(EVENT::LCRunHeader* run);

        virtual void processEvent(EVENT::LCEvent* evt);

        virtual void check(EVENT::LCEvent* evt);

        virtual void end();


    protected:
        /* the place for protected and private member data and functions */
        std::string _inputColName; ///< Name of the input collection
        std::string _splitHitsColName; ///< Name of the collection with split hits
        std::string _pulsesInSplitHitsColName; ///< Name of the input collection
        std::string _outputColName; ///< Name of the out tracks collection
        std::string _tpcConditionsColName; ///< Name of the TPCConditions collection (for vDrift)

        float _vDrift; ///< The drift velocity

};
} // namespace marlintpc
#endif // HITSPLITTERPROCESSOR_H
