#ifndef LINEARTRACKREGRESSION_H
#define LINEARTRACKREGRESSION_H

namespace EVENT
{
class Track;
}

namespace marlintpc
{


/** Helper class to perform a linear regression on a track.
 *
 *  For straight lines the linear regression is equivalent to
 *  a \f$\chi^2\f$ minimisation of the track parameters,
 *  but much faster because it is calcualted analytically.
 */

class LinearTrackRegression
{
    private:
        LinearTrackRegression();

    public:
        /** A nested struct which is used as return value of the regression.
         *  Slope1 and intersept1 are the track parameters in the xy-plane.\n
         *  Slope2 and intersept2 are the track parameters in the xz- or yz-plane, respectively.
         *
         *  Depending on the orientation of the track this could be\n
         *  \n
         *  y = slope1 * x + intersept1\n
         *  z = slope2 * x + intersept2\n
         *  \n
         *            or\n
         *  \n
         *  x = slope1 * y + intersept1\n
         *  z = slope2 * y + intersept2\n
         */
        struct StraightLine
        {
            float slope1, intersept1, slope2, intersept2;
        };

        /** gives back parameters for \n
         *  x = slope1 * y + intersept1\n
         *  z = slope2 * y + intersept2\n
         */
        static LinearTrackRegression::StraightLine doRegressionX(EVENT::Track const *track);

        /** gives back parameters for \n
         *  y = slope1 * x + intersept1\n
         *  z = slope2 * x + intersept2\n
         */
        static LinearTrackRegression::StraightLine doRegressionY(EVENT::Track const *track);
};

} // namespace marlintpc


#endif // LINEARTRACKREGRESSION_H
