#ifndef SIMPLE_MIMINIZER_RESIDUALS_FUNCTION_H
#define SIMPLE_MINIMIZER_RESIDUALS_FUNCTION_H 1

#include <Minuit2/FCNBase.h>
#include <EVENT/Track.h>
#include <TrackFitterBase.h>

namespace marlintpc
{

/**
 * The Minuit2 fitter function to calculate the squared sum of the residuals.
 * Derived from FCNBase. This corresonds to a \f$\chi^2\f$ fit with all errors set to one.
 * Advantage: No covariance matrix of the hits is required.
 */

class SimpleMinimizerResidualsFunction : public ROOT::Minuit2::FCNBase
{

    public:
        SimpleMinimizerResidualsFunction(const EVENT::Track * const t, TrackFitterBase const * f) ;
        ~SimpleMinimizerResidualsFunction() {}
        virtual double Up() const
        {
            return theErrorDef;
        }
        virtual double operator()(const std::vector<double>&) const;
        void SetErrorDef(double def)
        {
            theErrorDef = def;
        }

    private:

        const EVENT::Track * const track;
        TrackFitterBase const * fitter;
        double theErrorDef;
};

}//namespace marlintpc

#endif //SIMPLE_MINIMIZER_RESIDUALS_FUNCTION_H
