#ifndef TPCHITDOMAIN_H
#define TPCHITDOMAIN_H

#include "TPCHitMap.h"
#include "TMatrixD.h"

namespace marlintpc
{
/** This is a class to define a domain on the TPC readout plane.
*  The domain is specified by module and row.
 *
 *  @author Bo Li, Tsinghua University
 *  @author Keisuke Fujii, KEK
 */

class TPCHitDomain
{
    public:
        TPCHitDomain(const gear::TPCParameters*, int maxSkip);

        /** Set a domain next to hit
         */
        bool setDomain(const ModuleRowHit& hit, int step);

        /** Skip a domain. If the track gets to one module boundary,
         *  use the information last hit added into track to extrapolate the next module.
         */
        bool skipDomain(const ModuleRowHit& hit);

        /** Get Module ID
         */
        inline int     getModule()
        {
            return _mod;
        }

        /** Get row index
         */
        inline int     getRow()
        {
            return _row;
        }

        /** Reset the skipping number
         */
        inline void    resetSkip()
        {
            _skip = _MAX_SKIP;
        }

        /** Called by setDomain() and skipDomain().
         *  It attemps to deal with sitiation when track go through module.
         */
        bool   atBoundary(const ModuleRowHit& hit);

    private:
        /** The row index
         */
        int    _row;

        /** The Module ID
         */
        int    _mod;

        /** The number of skipping row
         */
        int    _skip;

        /** The TPC geometry
         */
        const  gear::TPCParameters* _tpcParameters;

        /** Maximum track skipping row
         */
        const  int _MAX_SKIP;
};

}
#endif  //TPCHITDOMAIN_H
