#ifndef TPCHITMAP_H
#define TPCHITMAP_H

#include <map>
#include <list>
#include <string>

#include <gear/TPCParameters.h>
#include <EVENT/LCEvent.h>
#include <IMPL/TrackerHitImpl.h>

typedef std::pair< std::pair<int, int>, std::list<EVENT::TrackerHit*>::iterator >  ModuleRowHit;///< A type which combins moudle, row and hit

namespace marlintpc
{

class TPCHitDomain;

/** This is hits management class for TPC.
 *
 *  @author Bo Li, Tsinghua University
 *  @author Keisuke Fujii, KEK
 */
class TPCHitMap
{

    public:
        TPCHitMap(const gear::TPCParameters* tp, const std::string inputName, 
		  const std::map<int, int>* moduleOrder);

        ~TPCHitMap();

        /** Fill hits to this data container and arrange them by module and row
         */
        void                fillEventHits(EVENT::LCEvent* evt);

        /** Remove hits from the container
         */
        void                removeHits(std::list<ModuleRowHit>* hitlist);

        /** Remove a hit from the container
         */
        void                removeHit(ModuleRowHit hit);

        /**
         */
        void                update();

        /** Clear all hits
         */
        void                clear();

        /** Is the container empty?
         */
        inline bool         isEmpty()
        {
            return _empty;
        }

        /** Get hit size of container
         */
        int                 size();

        /** Get the first hit from the container
         */
        ModuleRowHit           getFirstHit();

        /** Get Hits from a TPC domain
         */
	std::list<ModuleRowHit>     getHitsInDomain(TPCHitDomain &domain);

        static EVENT::TrackerHit*   getTrackerHit(ModuleRowHit hit);
        static const double*        getHitPosition(ModuleRowHit hit);
        inline static int           getModule(ModuleRowHit hit)
        {
            return (hit.first).first;
        }
        inline static int           getRow(ModuleRowHit hit)
        {
            return (hit.first).second;
        }

        inline const gear::TPCParameters* getGeometry()
        {
            return _tpcParameters;
        }

    private:
        /** The hits container
         */
	std::list<EVENT::TrackerHit*>**      _hitList;

        /** Current module
         */
        int                      _curModule;

        /** The module order
         */
        const std::map<int, int>*      _moduleOrder;

        const gear::TPCParameters*     _tpcParameters;

	std::string                   _hitInputColName;

        /** It is true when the container is empty
         */
        bool                     _empty;

        /** Record which (module, row) has hit
         */
	std::list< std::pair<int, int> >   _recorder;
};


}
#endif //TPCHITMAP_H
