#include "SimpleChiSquareResidualsFunction.h"
#include <cmath>
#include <streamlog/streamlog.h>

namespace marlintpc
{
SimpleChiSquareResidualsFunction::SimpleChiSquareResidualsFunction(const EVENT::Track * const t,
                                                                   TrackFitterSimpleChiSquare const * f,
                                                                   double zAnode)
        : _track(t) , _fitter(f), _theErrorDef(1.), _zAnode(zAnode)
{
    streamlog_out(DEBUG2) << "SimpleChiSquareResidualsFunction constructor " << std::endl;
}

double SimpleChiSquareResidualsFunction::operator()(const std::vector<double>&trackParams) const
{
    double chiSquare = 0;
    IMPL::TrackImpl trackWithTestParameters;
    trackWithTestParameters.setD0(trackParams[0]);
    trackWithTestParameters.setPhi(trackParams[1]);
    trackWithTestParameters.setOmega(trackParams[2]);
    trackWithTestParameters.setTanLambda(trackParams[3]);
    trackWithTestParameters.setZ0(trackParams[4]);

    streamlog_out(DEBUG1) << "SimpleChiSquareResidualsFunction: "
    << " ; d0: " << trackParams[0]
    << " ; phi: " << trackParams[1]
    << " ; omega: " << trackParams[2]
    << " ; tamLambda: " << trackParams[3]
    << " ; z0: " << trackParams[4]
    << std::endl;



    for (TrackerHitVec::const_iterator hitIter = _track->getTrackerHits().begin() ;
            hitIter < _track->getTrackerHits().end() ; hitIter++)
        {
            EVENT::DoubleVec residuals = _fitter->calculateResiduals(*hitIter, &trackWithTestParameters);


            double driftDistance;
            if (_zAnode)
                {
                    driftDistance = _zAnode  - fabs((*hitIter)->getPosition()[2]);
                    if (driftDistance < 0)
                        {
                            streamlog_out(ERROR2) << "z-Coordinate is larger than drift dristance,"
                            << " aborting chisquare calculation!" << std::endl;
                            chiSquare = 0;
                            break;
                        }
                }
            else
                {
                    driftDistance = (*hitIter)->getPosition()[2];
                    if (driftDistance < 0)
                        {
                            streamlog_out(ERROR2) << "negative drift distance,"
                            << " aborting chisquare calculation!" << std::endl;
                            chiSquare = 0;
                            break;
                        }
                }

            double sigma_trans = sqrt(_fitter->getTransDefocussing() * _fitter->getTransDefocussing()
                                      + _fitter->getTransDiffusionCoef() * _fitter->getTransDiffusionCoef()
                                      * driftDistance);

            // FIXME: calculate real chiSquare taking into account the errors
            chiSquare += residuals[0] * residuals[0] / (sigma_trans * sigma_trans);

            if (!_fitter->getNoZFitFlag())   // z direction only if not disabled
                {

                    double sigma_long = sqrt(_fitter->getLongDefocussing() * _fitter->getLongDefocussing()
                                             + _fitter->getLongDiffusionCoef() * _fitter->getLongDiffusionCoef()
                                             * driftDistance);
                    streamlog_out(DEBUG0) << "SimpleChiSquareResidualsFunction: sigmalong"  << sigma_long
                    << " ; sigmatrans " << sigma_trans << std::endl;
                    streamlog_out(DEBUG0) << "SimpleChiSquareResidualsFunction: resi"  << residuals[0]
                    << " ; " << residuals[1] << std::endl;

                    chiSquare += residuals[1] * residuals[1] / (sigma_long * sigma_long) ;
                }
        }

    streamlog_out(DEBUG1) << "SimpleChiSquareResidualsFunction: chiSquare is " << chiSquare << std::endl;
    return chiSquare;
}


}//namespace marlintpc
