
// STL
#include <vector>
#include <map>
#include <algorithm>
#include <iomanip>
#include <cassert>
#include <fstream>

// LCIO
#include "EVENT/LCIO.h"
#include "EVENT/LCCollection.h"
#include "EVENT/TrackerHit.h"
#include "IMPL/LCCollectionVec.h"
#include "IMPL/TrackImpl.h"
#include <IMPL/LCFlagImpl.h>
#include <IMPL/TrackerPulseImpl.h>
#include <streamlog/streamlog.h>

// CLHEP
#include "CLHEP/Vector/ThreeVector.h"

// GEAR
#include "gear/GEAR.h"
#include "gear/TPCParameters.h"
#include "gear/TPCModule.h"
#include "gear/PadRowLayout2D.h"
#include "gearxml/GearXML.h"

// Marlin
#include "marlin/Global.h"
#include "TrackConnection.h"

using namespace marlin;
using namespace lcio;

namespace marlintpc
{


TrackConnection::TrackConnection()
{
}


TrackConnection::~TrackConnection()
{
}


void TrackConnection::mergeTracks(IMPL::TrackImpl*& mergedTrack, std::multimap<int, IMPL::TrackImpl*> &tracksByModuleID)
{
    EVENT::TrackerHitVec trackhit = mergedTrack->getTrackerHits();

    streamlog_out(DEBUG0) << "Start Sorting" << std::endl;
    std::stable_sort(trackhit.begin(), trackhit.end(), Compare());

    //get ModuleID
    TrackerHitVec::const_reverse_iterator hitsInSeed = mergedTrack->getTrackerHits().rbegin();
    TrackerHit* lastHit = *hitsInSeed;
    const double* xv = lastHit->getPosition();
    gear::TPCParameters const &theTPCParameters = marlin::Global::GEAR->getTPCParameters();
    const gear::TPCModule &currentModule = theTPCParameters.getNearestModule(xv[0], xv[1]);
    const int currentModuleID = currentModule.getModuleID();
    streamlog_out(MESSAGE4) << "currentModuleID = " << currentModuleID << std::endl;

    //nearest Module assign
    int nearestModuleID = -1;
    switch (currentModuleID)
        {
        case 0:
            nearestModuleID = 3;
            break;
        case 3:
            nearestModuleID = 5;
            break;
        default:
            break;
        }

#if 1
    // Get Hit Position for Connection Condition
    TrackerHitVec::const_reverse_iterator mHit = trackhit.rbegin();
    mHit++;
    _x3 = (*mHit)->getPosition();
    mHit++;
    //_x2  = (*mHit)->getPosition();
    mHit++;
    _x1 = (*mHit)->getPosition();

    streamlog_out(DEBUG) << "Connect Position" << std::endl;
    streamlog_out(DEBUG) << "_x1 = ( " << _x1[0] << " , " << _x1[1] << " , " << _x1[2] << " )" << std::endl;
    streamlog_out(DEBUG) << "_x3 = ( " << _x3[0] << " , " << _x3[1] << " , " << _x3[2] << " )" << std::endl;
#endif

    if (nearestModuleID > 0)
        {
            streamlog_out(MESSAGE4) << "nearestModuleID = " << nearestModuleID << std::endl;

            bool found = false;

            std::multimap<int, IMPL::TrackImpl*>::iterator nextSegment;
            streamlog_out(DEBUG) << "Start loop nextSegment" << std::endl;
            //loop over Tracks in map
            for (nextSegment = tracksByModuleID.lower_bound(nearestModuleID);
                    nextSegment != tracksByModuleID.upper_bound(nearestModuleID);
                    nextSegment++)
                {
                    streamlog_out(DEBUG) << "Start if connectSegment" << std::endl;
                    if (connectSegment(mergedTrack, nextSegment))
                        {
                            found = true;
                            nextSegment->second->ext<UsedFlag>() = 1;
                            streamlog_out(DEBUG) << "connectSegment true" << std::endl;
                            break;
                        }
                }
            if (found)
                {
                    mergeTracks(mergedTrack, tracksByModuleID);
                }
        }
    streamlog_out(DEBUG) << "Merge Complete!!" << std::endl;
    streamlog_out(DEBUG) << "MergedTrack has " << (int)distance(mergedTrack->getTrackerHits().begin(), mergedTrack->getTrackerHits().end())
    << " Hits." << std::endl;

    EVENT::TrackerHitVec mergedTrackHit = mergedTrack->getTrackerHits();
    std::stable_sort(mergedTrackHit.begin(), mergedTrackHit.end(), Compare());

    //Renewed mergedTrack to append hits because of "const"
    delete mergedTrack;
    mergedTrack = new IMPL::TrackImpl;
    for (TrackerHitVec::const_iterator hitIter = mergedTrackHit.begin(); hitIter < mergedTrackHit.end(); hitIter++)
        {
            mergedTrack->addHit(*hitIter);
        }
}


bool TrackConnection::connectSegment(IMPL::TrackImpl* mergedTrack, std::multimap<int, IMPL::TrackImpl*>::iterator nextSegment)
{
    streamlog_out(DEBUG) << "Start connectSegment function " << std::endl;

    EVENT::TrackerHitVec tempTrackerHits = mergedTrack->getTrackerHits();
    if (tempTrackerHits.size() < 6)
        {
            streamlog_out(WARNING2) << "Less than 6 hits in track candidates. "
            << "Check your settings of the track finder!" << std::endl;
            return false;
        }

#if 1

    TrackerHitVec::const_iterator nHit = (nextSegment->second)->getTrackerHits().begin();
    const double* _xnext = (*nHit)->getPosition();

    streamlog_out(DEBUG) << "Connect Position" << std::endl;
    streamlog_out(DEBUG) << "_x1 = ( " << _x1[0] << " , " << _x1[1] << " , " << _x1[2] << " )" << std::endl;
    streamlog_out(DEBUG) << "_x3 = ( " << _x3[0] << " , " << _x3[1] << " , " << _x3[2] << " )" << std::endl;
    streamlog_out(DEBUG) << "_xnext = ( " << _xnext[0] << " , " << _xnext[1] << " , " << _xnext[2] << " )" << std::endl;

#if 0 // 2D
    CLHEP::Hep3Vector xv3(_x3[0], _x3[1]);
    CLHEP::Hep3Vector xv1(_x1[0], _x1[1]);
    CLHEP::Hep3Vector xvnext(_xnext[0], _xnext[1]);
#else // 3D
    CLHEP::Hep3Vector xv3(_x3[0], _x3[1], _x3[2]);
    CLHEP::Hep3Vector xv1(_x1[0], _x1[1], _x1[2]);
    CLHEP::Hep3Vector xvnext(_xnext[0], _xnext[1], _xnext[2]);
#endif

    CLHEP::Hep3Vector xvn13 = (xv3 - xv1).unit();
    CLHEP::Hep3Vector xvn1next = (xvnext - xv1).unit();
    double costheta = xvn13 * xvn1next;

#if 0
    if (costheta == 0)
        {
            std::ofstream cos0_out("CosTheta_0.txt", std::ios::out | std::ios::app);
            if (!cos0_out)
                {
                    std::cout << "Can't open file!! \n";
                }
            cos0_out << "costheta = 0 !" << std::endl;
            cos0_out << "_x1 = ( " << _x1[0] << " , " << _x1[1] << " , " << _x1[2] << " )" << std::endl;
            cos0_out << "_x3 = ( " << _x3[0] << " , " << _x3[1] << " , " << _x3[2] << " )" << std::endl;
            cos0_out << "_xnext = ( " << _xnext[0] << " , " << _xnext[1] << " , " << _xnext[2] << " )" << std::endl << std::endl;
        }

    std::ofstream fout("CosTheta.txt", std::ios::out | std::ios::app);
    if (!fout)
        {
            std::cout << "Can't open file!! \n";
        }
    fout << costheta << std::endl;
#endif

    if (costheta > _coneAngle)
        {
            connect(mergedTrack, nextSegment);
            return true;
        }
    else
        {
            return false;
        }

#else

    if (true)
        {
            connect(mergedTrack, nextSegment);
            return true;
        }
    else
        {
            return false;
        }
#endif

}


void TrackConnection::connect(IMPL::TrackImpl* mergedTrack, std::multimap<int, IMPL::TrackImpl*>::iterator nextSegment)
{
    streamlog_out(DEBUG) << "Start connect function " << std::endl;

    IMPL::TrackImpl* tempTrack = nextSegment->second;
    EVENT::TrackerHitVec nextSegmentTrackerHits = tempTrack->getTrackerHits();
    for (EVENT::TrackerHitVec::iterator hit = nextSegmentTrackerHits.begin();
            hit < nextSegmentTrackerHits.end(); hit++)
        {
            mergedTrack->addHit(*hit);
        }

    streamlog_out(DEBUG) << "Merged Success!" << std::endl;
    streamlog_out(DEBUG) << "MergedTrack has " << (int)distance(mergedTrack->getTrackerHits().begin(), mergedTrack->getTrackerHits().end())
    << " Hits." << std::endl;

}


void TrackConnection::setConeAngle(double angle)
{
  _coneAngle = angle;

}

} // namespace marlintpc
