#ifndef CHARGESIGNAL_H
#define CHARGESIGNAL_H 1

// stl
#include <string>

// LCIO
#include <EVENT/LCGenericObject.h>

namespace marlintpc {

/** ChargeSignal : A parameterized signal on a pad
 *
 *    @author J. Abernathy (UVic)<br>
 */
class ChargeSignal : public EVENT::LCGenericObject {

public:
	/** constructor
	 */
	ChargeSignal(int, int, int);


	/** gets the number of electrons
	 */
	int get_electron_count( void ) const;


	/** add an electron to the signal at a specified time
	 */
	void add_electron( const double );


	/** gets the unique id of this charge signal
	 */
	int get_charge_id( void ) const;


	/** gets the pad id of this signal
	 */
	int get_pad_id( void ) const;


	/** gets the module id of this charge signal
	 */
	int get_module_id( void ) const;


	/** gets the variance in the time
	 */
	double get_time_variance( void ) const;


	/** gets the mean time of the signal
	 */
	double get_mean_time( void ) const;


	/** returns the number of integers used
	 */
	int getNInt() const;


	/** returns the number of floats used
	 */
	int getNFloat() const;


	/** returns the number of doubles used
	 */
	int getNDouble() const;


	/** returns an int value at a given index
	 */
	int getIntVal( int ) const;


	/** returns a float value at a given index
	 */
	float getFloatVal( int ) const;


	/** returns a double value at a given index
	 */
	double getDoubleVal( int ) const;


	/** returns where or not the number of parameters change during runtime
	 */
	bool isFixedSize() const;


	/** returns the typename of the class
	 */
	const std::string getTypeName( void ) const;


	/** returns the type of data in the generic object
	 */
	const std::string getDataDescription( void ) const;

protected:

private:

	/** the charge id of the signal
	 */
	int _charge_id;


	/** the id of the pad which the signal belongs to
	 */
	int _pad_id;


	/** the module id which the pad belongs to
	 */
	int _module_id;

	/** the number of electrons in the charge signal
	 */
	int _electron_count;

	
	/** the mean arrival time of the electrons in the charge signal
	 */
	double _mean_time;


	/** the unadjusted variance of the arrival time of the electrons
	 */
	double _S;


	/** sets the variance in the time
	 */
	void set_time_variance( const double );


	/** sets the module id of this charge signal
	 */
	void set_module_id( const int );


	/** sets the mean time of the signal
	 */
	void set_mean_time( const double );
};

} // namespace marlintpc

#endif // CHARGESIGNAL_H
