/*
 * AdvancedAltroConverter.cpp
 *
 *  Created on: Aug 20, 2012
 *      Author: caiazza
 */

#include <iostream>
#include <string>
#include <sstream>
#include <fstream>
#include <vector>
#include <list>
#include <getopt.h>
#include <stdlib.h>

//MARLINTPC
#include "TAltroConverter.hh"
#include "TAltroPedestalConverter.hh"
#include "ADCChannelMapping.h"

//LCIO
#include "lcio.h"
#include "UTIL/LCSplitWriter.h"
#include "EVENT/LCEvent.h"
#include "EVENT/LCCollection.h"

//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"

//ilcutil
#define DONT_USE_STREAMLOG
#include "streamlog/streamlog.h"

//Forward declaring functions to be used in the main. They are defined after main
void
PrintUsage();

std::vector<std::ifstream*>
CreateInputStreams(std::string& inputfile);

std::string
GenerateOutFileName(std::string& inputfile);

int
main(int argc, char* argv[])
{
    streamlog_out(MESSAGE) << "Executing the Advanced conversion program for the ALTRO data" << std::endl;
    if(argc == 1)
    {
        PrintUsage();
        exit(1);
    }

    // Defining the options that must be read from the command line
    // Options necessary for the simple data conversion
    std::string InputFileName = "";
    std::string OutputFileName = "";
    std::string DataCollectionName = "";
    const std::string DEFAULTDATACOLNAME = "AltroRawData";
    std::string DetectorName = "";
    const std::string DEFAULTDETECTORNAME = "LPTPC";
    std::string RunDescription = "";
    const std::string DEFAULTRUNDESCRIPTION = "Test beam run";
    bool OverrideZeroSuppression = false;
    int ZeroSuppression;

    bool useCondDB = false;
    std::string DBInit = "";
    std::string GenericElectronicsDBFolder = "";
    const std::string DEFAULTGENDBFOLDER = "/LP1/T24_1/DESY/Grid_GEM_Module_V.2/ADCElectronicsParameters";
    std::string ExtendedElectronicsDBFolder = "";
    const std::string DEFAULTEXTDBFOLDER = "/LP1/T24_1/DESY/Grid_GEM_Module_V.2/AltroExtElectronicsParameters";

    bool storeChanMap = false;
    std::string ChannelMapLCIOFile = "";
    std::string ChannelMapColName = "";
    const std::string DEFAULTCHANMAPCOLNAME = tpcconddata::ADCChannelMapping::getDefaultColName();
    std::string ChannelMapDBFolder = "";
    const std::string DEFAULTCHANMAPDBFOLDER = "/LP1/T24_1/DESY/Grid_GEM_Module_V.2/channelmapping";

    bool StorePed2DB = false;
    std::vector<std::string> PedestalFileNames;
    std::string PedestalDBHWFolder = "";
    const std::string DEFAULTPEDDBHWFOLDER = "/LP1/T24_1/DESY/Grid_GEM_Module_V.2/pedestal_readout";
    std::string PedestalDBGearFolder = "";
    bool RemapPedestals = false;
    const std::string DEFAULTPEDDBGEARFOLDER = "/LP1/T24_1/DESY/Grid_GEM_Module_V.2/pedestal_gear";

    // Interpreting the command line
    static struct option long_options[] =
        {
            { "in", required_argument, 0, 'I' },
            { "out", required_argument, 0, 'O' },
            { "colname", required_argument, 0, 'C' },
            { "detname", required_argument, 0, 'N' },
            { "rundesc", required_argument, 0, 'T' },
            { "zsuppr", required_argument, 0, 'Z' },
            { "dbinit", required_argument, 0, 'D' },
            { "gendb", required_argument, 0, 'G' },
            { "extdb", required_argument, 0, 'E' },
            { "mapfile", required_argument, 0, 'M' },
            { "mapdb", optional_argument, 0, 'm' },
            { "mapcol", required_argument, 0, 'H' },
            { "pedfile", required_argument, 0, 'P' },
            { "peddbhw", required_argument, 0, 'p' },
            { "remap", optional_argument, 0, 'R' }
        };

    int OptionId;
    int OptionIndex;

    while(( OptionId = getopt_long(argc, argv, "I:O:C:N:T:Z:D:G:E:M:m::H:P:p::R::", long_options, &OptionIndex) ) != -1)
    {
        switch(OptionId)
        {
        case 'I':
        {
            InputFileName = std::string(optarg);
            streamlog_out(MESSAGE) << "Base file name to convert this run: " << std::string(optarg) << std::endl;
            break;
        }
        case 'O':
        {
            OutputFileName = std::string(optarg);
            streamlog_out(MESSAGE) << "Output file name provided by the user: " << std::string(optarg) << std::endl;
            break;
        }
        case 'C':
        {
            DataCollectionName = std::string(optarg);
            streamlog_out(MESSAGE) << "Data Collection Name set to " << DataCollectionName << std::endl;
            break;
        }
        case 'N':
        {
            DetectorName = std::string(optarg);
            streamlog_out(MESSAGE) << "Detector Name set to " << DetectorName << std::endl;
            break;
        }
        case 'T':
        {
            RunDescription = std::string(optarg);
            streamlog_out(MESSAGE) << "Run Description set to " << RunDescription << std::endl;
            break;
        }
        case 'Z':
        {
            OverrideZeroSuppression = true;
            std::string Arg(optarg);
            std::stringstream oss(Arg);
            oss >> ZeroSuppression;
            streamlog_out(MESSAGE) << "Overriding the zero suppression data to be stored to: " << ZeroSuppression
                << std::endl;
            break;
        }
        case 'D':
        {
            DBInit = std::string(optarg);
            useCondDB = true;
            streamlog_out(MESSAGE) << "DB Initialization string is: " << DBInit << std::endl;
            break;
        }
        case 'G':
        {
            GenericElectronicsDBFolder = std::string(optarg);
            streamlog_out(MESSAGE) << "DB Folder for the generic electronics info is: " << GenericElectronicsDBFolder << std::endl;
            break;
        }
        case 'E':
        {
            ExtendedElectronicsDBFolder = std::string(optarg);
            streamlog_out(MESSAGE) << "DB Folder for the extended electronics info is: " << ExtendedElectronicsDBFolder << std::endl;
            break;
        }
        case 'M':
        {
            ChannelMapLCIOFile = std::string(optarg);
            streamlog_out(MESSAGE) << "Channel Mapping file set to: " << ChannelMapLCIOFile << std::endl;
            break;
        }
        case 'm':
        {
            storeChanMap = true;
            if(optarg)
            {
                ChannelMapDBFolder = std::string(optarg);
                streamlog_out(MESSAGE) << "DB Folder for the channel mapping info is: " << ChannelMapDBFolder << std::endl;
            }
            break;
        }
        case 'H':
        {
            ChannelMapColName = std::string(optarg);
            streamlog_out(MESSAGE) << "Collection Name for the channel mapping data is: " << ChannelMapColName << std::endl;
            break;
        }
        case 'P':
        {
            StorePed2DB = true;
            PedestalFileNames.push_back(std::string(optarg));
            streamlog_out(MESSAGE) << "Adding a file to the pedestals to load to the database: " << optarg << std::endl;
            break;
        }
        case 'p':
        {
            PedestalDBHWFolder = std::string(optarg);
            streamlog_out(MESSAGE) << "DB Folder for the pedestals (hardware mapping) is: " << PedestalDBHWFolder << std::endl;
            break;
        }
        case 'R':
        {
            RemapPedestals = true;
            streamlog_out(MESSAGE) << "The pedestals will be saved on the database remapped to GEAR id as well" << std::endl;
            if(optarg)
            {
                PedestalDBGearFolder =  std::string(optarg);
                streamlog_out(MESSAGE) << "DB Folder for the pedestals (GEAR mapping) is: " << PedestalDBGearFolder << std::endl;
            }
            break;
        }
        case '?':
        {
            PrintUsage();
            exit(1);
        }
        }
    }

    //Checking the command line arguments and setting default values
    if(InputFileName == "")
    {
        streamlog_out(ERROR) << "ERROR:This application requires at least one input file to convert." << std::endl;
        PrintUsage();
        exit(1);
    }

    if(OutputFileName == "")
    {
        streamlog_out(MESSAGE) << "No name was provided for the output file. Generating one automatically. " << std::endl;
        OutputFileName = GenerateOutFileName(InputFileName);
        streamlog_out(MESSAGE) << "The output will be written to " << OutputFileName << std::endl;
    }

    if(DataCollectionName == "")
    {
        streamlog_out(MESSAGE) << "No name was provided data collection, using the default one. " << std::endl;
        DataCollectionName = DEFAULTDATACOLNAME;
        streamlog_out(MESSAGE) << "The output will be written to the collection " << DataCollectionName << std::endl;
    }

    if(DetectorName == "")
    {
        streamlog_out(MESSAGE) << "No detector name was provided, using the default one. " << std::endl;
        DetectorName = DEFAULTDETECTORNAME;
        streamlog_out(MESSAGE) << "Detector Name set to " << DetectorName << std::endl;
    }

    if(RunDescription == "")
    {
        streamlog_out(MESSAGE) << "No run description was provided, using the default one. " << std::endl;
        RunDescription = DEFAULTRUNDESCRIPTION;
        streamlog_out(MESSAGE) << "Run description set to " << RunDescription << std::endl;
    }

    if(GenericElectronicsDBFolder == "")
    {
        streamlog_out(MESSAGE) << "No folder for the generic electronic info was provided, using the default one. " << std::endl;
        GenericElectronicsDBFolder = DEFAULTGENDBFOLDER;
        streamlog_out(MESSAGE) << "Generic electronic info DB folder set to: " << GenericElectronicsDBFolder << std::endl;
    }

    if(ExtendedElectronicsDBFolder == "")
    {
        streamlog_out(MESSAGE) << "No folder for the extended electronic info was provided, using the default one. " << std::endl;
        ExtendedElectronicsDBFolder = DEFAULTEXTDBFOLDER;
        streamlog_out(MESSAGE) << "Extended electronic info DB folder set to: " << ExtendedElectronicsDBFolder << std::endl;
    }

    if(ChannelMapDBFolder == "")
    {
        streamlog_out(MESSAGE) << "No folder for the channel mapping info was provided, using the default one. " << std::endl;
        ChannelMapDBFolder = DEFAULTCHANMAPDBFOLDER;
        streamlog_out(MESSAGE) << "Channel Mapping info DB folder set to: " << ChannelMapDBFolder << std::endl;
    }

    if(ChannelMapColName == "")
    {
        streamlog_out(MESSAGE) << "No name for the channel mapping collection was provided, using the default one. " << std::endl;
        ChannelMapColName = DEFAULTCHANMAPCOLNAME;
        streamlog_out(MESSAGE) << "Channel mapping collection name set to: " << ChannelMapColName << std::endl;
    }

    if(PedestalDBHWFolder == "")
    {
        streamlog_out(MESSAGE) << "No DB folder for the pedestal data (hardware mapping) was provided, using the default one. " << std::endl;
        PedestalDBHWFolder = DEFAULTPEDDBHWFOLDER;
        streamlog_out(MESSAGE) << "Pedestal data (hardware mapping) DB folder set to: " << PedestalDBHWFolder << std::endl;
    }

    if(PedestalDBGearFolder == "")
    {
        streamlog_out(MESSAGE) << "No DB folder for the pedestal data (GEAR mapping) was provided, using the default one. " << std::endl;
        PedestalDBGearFolder = DEFAULTPEDDBGEARFOLDER;
        streamlog_out(MESSAGE) << "Pedestal data (GEAR mapping) DB folder set to: " << PedestalDBGearFolder << std::endl;
    }

    if(RemapPedestals && (ChannelMapLCIOFile == ""))
    {
        streamlog_out(ERROR) << "ERROR:This application requires at least one input file to convert." << std::endl;
        PrintUsage();
        exit(1);
    }

    streamlog_out(MESSAGE) << "\n\n All parameters have been correctly set. Starting the conversion.\n" << std::endl;


    // Creating the local variables for the main routine
    std::vector<std::ifstream*> InputStreams = CreateInputStreams(InputFileName);
    lcio::LCWriter* lciowriter = 0;
    lcio::LCReader* lcreader = 0;
    lccd::DBInterface* DBGenericInfo = 0;
    lccd::DBInterface* DBExtendedInfo = 0;
    lccd::DBInterface* DBChanMap = 0;
    lccd::DBInterface* DBPedestal = 0;
    lcio::LCEvent* evt = 0;

    try
    {
        lciowriter = new lcio::LCSplitWriter(lcio::LCFactory::getInstance()->createLCWriter(), 2040109465);
        lciowriter->open(OutputFileName);

        marlintpc::TAltroConverter Converter(InputStreams, lciowriter);
        Converter.SetDataCollectionName(DataCollectionName);
        Converter.SetDetectorName(DetectorName);
        Converter.SetRunDescription(RunDescription);
        if(OverrideZeroSuppression)
        {
            Converter.SetZSThresholdOverride(ZeroSuppression);
        }
        streamlog_out(MESSAGE) << "\n\n Starting the decoding of the raw data.\n" << std::endl;
        Converter.Decode();
        Converter.Print(std::cout);

        streamlog_out(MESSAGE) << "\n\n Decoding COMPLETE.\n" << std::endl;

        lciowriter->close();
        delete lciowriter;
        lciowriter = 0;

        std::vector<std::ifstream*>::iterator itStreams = InputStreams.begin();
        for(; itStreams!= InputStreams.end(); ++itStreams)
        {
            (*itStreams)->close();
            delete (*itStreams);
            (*itStreams) = 0;
        }

        lccd::LCCDTimeStamp RunStart=Converter.RunStart();
        lccd::LCCDTimeStamp RunEnd=Converter.RunEnd() + 1000000000;

        if(useCondDB)
        {
            DBGenericInfo = new lccd::DBInterface(DBInit, GenericElectronicsDBFolder, true);
            DBExtendedInfo = new lccd::DBInterface(DBInit, ExtendedElectronicsDBFolder, true);
            Converter.ToDB(DBGenericInfo, DBExtendedInfo);
            delete DBGenericInfo;
            DBGenericInfo = 0;
            delete DBExtendedInfo;
            DBExtendedInfo = 0;

            streamlog_out(MESSAGE) << "\n\n Storage of the electronics information on DB COMPLETE.\n" << std::endl;
        }

        if(StorePed2DB)
        {
            DBPedestal = new lccd::DBInterface(DBInit, PedestalDBHWFolder, true);
            marlintpc::TAltroPedestalConverter PedestalConverter;
            PedestalConverter.Convert(PedestalFileNames);
            PedestalConverter.ToDB(*DBPedestal, RunStart, RunEnd);
            delete DBPedestal;
            DBPedestal = 0;
            streamlog_out(MESSAGE) << "\n\n Storage of the pedestal with hardware mapping COMPLETE.\n" << std::endl;
        }

        if(ChannelMapLCIOFile != "")
        {
            lcreader = lcio::LCFactory::getInstance()->createLCReader();
            lcreader->open(ChannelMapLCIOFile);
            evt = lcreader->readNextEvent();
            lcio::LCCollection* ChanMapCol = evt->getCollection(ChannelMapColName);
            streamlog_out(MESSAGE) << "\n\n Channel mapping collection FOUND.\n" << std::endl;

            if(StorePed2DB && RemapPedestals)
            {
                DBPedestal = new lccd::DBInterface(DBInit, PedestalDBGearFolder, true);
                marlintpc::TAltroPedestalConverter PedestalConverter;
                PedestalConverter.Remap(ChanMapCol);
                PedestalConverter.Convert(PedestalFileNames);
                PedestalConverter.ToDB(*DBPedestal, RunStart, RunEnd);
                delete DBPedestal;
                DBPedestal = 0;
                streamlog_out(MESSAGE) << "\n\n Storage of the pedestal with GEAR mapping COMPLETE.\n" << std::endl;
            }

            if(useCondDB && storeChanMap)
            {
                DBChanMap = new lccd::DBInterface(DBInit, ChannelMapDBFolder, true);
                DBChanMap->storeCollection(RunStart, RunEnd, ChanMapCol, "Channel Mapping information");
                delete DBChanMap;
                DBChanMap = 0;
                streamlog_out(MESSAGE) << "\n\n Storage of the channel mapping COMPLETE.\n" << std::endl;
            }

            ChanMapCol = 0;
            delete lcreader;
            lcreader = 0;
        }
    }
    catch(std::exception & e)
    {
        //Cleanup
        std::vector<std::ifstream*>::iterator itStreams = InputStreams.begin();
        for(; itStreams!= InputStreams.end(); ++itStreams)
        {
            (*itStreams)->close();
            delete (*itStreams);
            (*itStreams) = 0;
        }

        delete lciowriter;
        lciowriter = 0;
        delete lcreader;
        lcreader = 0;
        delete DBGenericInfo;
        DBGenericInfo = 0;
        delete DBExtendedInfo;
        DBExtendedInfo = 0;
        delete DBChanMap;
        DBChanMap = 0;
        delete DBPedestal;
        DBPedestal = 0;
        std::cout << e.what();
        exit(1);
    }

    streamlog_out(MESSAGE) << "Execution COMPLETE" << std::endl;
    exit(0);
}

void
PrintUsage()
{
    streamlog_out(MESSAGE) << "DESCRIPTION" << std::endl;
    streamlog_out(MESSAGE) << "\t\t This program converts altro raw data to LCIO data in the slcio format "
        << "and feed the relevant data for the first steps of the reconstruction "
        << "to the condition database. This data are the Electronics data, the Pedestals "
        << "and the channel mappings." << std::endl
        << "\t\t The minimal requirement of the program is one input file name. "
        << "If the -I option is not set the program assumes the last argument is the input file name" << std::endl;
    streamlog_out(MESSAGE) << "\t\t The input file must be the first of the data files produced by "
        << "the DAQ, which contains the start of run data block." << std::endl;
    streamlog_out(MESSAGE) << "\t\t If the input file names has the format <prefix>_0.dat, the program "
        << "will attempt to load all the other files with the same prefix and increasing counter "
        << "located in the same folder."
        << "Files that are not named sequentially as described above are considered as single " << "data files."
        << std::endl;
    streamlog_out(MESSAGE) << "\t\t If the option -O is not specified the output data "
        << "will be written to an slcio file whose name is obtained substituting the input file extension "
        << "or adding the .slcio extension in case of no extension in the input file" << std::endl;
    streamlog_out(MESSAGE) << "\t\t By default the program will only convert the raw data to a file. "
        << "If the user specifies the -D option it will also write the electronics data to the "
        << "database using the default collection names and folder unless otherwise specified "
        << "with the corresponding options. All the data for the condition database will be stored "
        << "with a validtity going from the run start to run end, as decoded from the raw data"<< std::endl;
    streamlog_out(MESSAGE) << "\t\t To additionally store the channel mapping information the user "
        << "shall provide the path to an lcio file containing the channel mapping information for the run "
        << "and the name of the collection where the data are stored. The user can also specify the database "
        << "folder where to store the data. Otherwise the default folder will be used. " << std::endl;
    streamlog_out(MESSAGE) << "\t\t Finally is also possible to store the pedestal data for the run with "
        << "validity interval going between the run start and run end as retrieved from the raw data file. "
        << "It's also possible to store contestually both the pedestal data with the standard hardware "
        << "mapping or the same data remapped by GEAR to the geometric identifiers. If the user doesn't "
        << "specify the folder and collection names the default ones will be used." << std::endl;
    streamlog_out(MESSAGE) << "\nSYNOPSIS" << std::endl;
    streamlog_out(MESSAGE) << "\t\t ConvertAltroraw2lcio INPUTFILE" << std::endl;
    streamlog_out(MESSAGE) << "\t\t ConvertAltroraw2lcio [options] INPUTFILE" << std::endl;
    streamlog_out(MESSAGE) << "\t\t ConvertAltroraw2lcio [options]" << std::endl;
    streamlog_out(MESSAGE) << "\nOPTIONS" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -I --in "
        << "\n\t\t\t Input file name. See the description above for more information" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -O --out"
        << "\n\t\t\t Output file name. If it's not provided it will be created using a standard scheme."
        << std::endl;
    streamlog_out(MESSAGE) << "\t\t -C --colname"
        << "\n\t\t\t Name of the collection where to store the raw data. Default is \"AltroRawData\"" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -N --detname"
        << "\n\t\t\t Value of the detector name to be set in the run header. Default is \"LPTPC\"" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -T --rundesc"
        << "\n\t\t\t Run description to be set in the run header. Default is \"Test beam run\"" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -Z --zsuppr"
        << "\n\t\t\t Overrides the value for the zero suppression threshold to be written in the condition data. "
        << "It's necessary to cope with the data format prior of 4.6 when this value was not retrievable from the data"<< std::endl;
    streamlog_out(MESSAGE) << "\t\t -D --dbinit"
        << "\n\t\t\t Enables the program to upload the electronics information to the database."
        << "It REQUIRES the CondDB initialization string as argument in the form "
        << "host:databasename:login:password[:port]." << std::endl;
    streamlog_out(MESSAGE) << "\t\t -G --gendb"
        << "\n\t\t\t Defines the path to the folder where to store the GenericElectronic Condition Data."
        << "Default: \"/LP1/T24_1/DESY/Grid_GEM_Module_V.2/ADCElectronicsParameters\" \n"
        << " Path format: /database/folder" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -E --extdb"
        << "\n\t\t\t Defines the path to the folder where to store the ExtendedElectronic Condition Data."
        << "Default: \"/LP1/T24_1/DESY/Grid_GEM_Module_V.2/AltroExtElectronicsParameters\" \n"
        << " Path format: /database/folder" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -M --mapfile"
        << "\n\t\t\t Defines the path to an lcio file containing the channel mapping information."
        << "If no collection name is provided the default one for the ADCChannelMapping type will be used to load the data"
        << std::endl;
    streamlog_out(MESSAGE) << "\t\t -m --mapdb"
        << "\n\t\t\t Defines the path to the folder where to store the Channel Mapping information. "
        << "If no argument is provided to this option it will simply activate the database storage using the default folder"
        << "Default: \"/LP1/T24_1/DESY/Grid_GEM_Module_V.2/channelmapping\" \n"
        << " Path format: /database/folder" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -H --mapcol"
        << "\n\t\t\t Defines the name of the collection containing the channel mapping data in the lcio file."
        << "Default: " << tpcconddata::ADCChannelMapping::getDefaultColName() << std::endl;
    streamlog_out(MESSAGE) << "\t\t -P --pedfile"
        << "\n\t\t\t Defines the path to a data file containing the pedestal information. "
        << "The text file should be formatted as: hwChannel intPedestal floatPedestal floatPedestalWidth integer"
        << "If no folder name is provided the default one will be used" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -p --peddbhw"
        << "\n\t\t\t Defines the DB folder where to store the Pedestal information using the hardware mapping. "
        << "Default: \"/LP1/T24_1/DESY/Grid_GEM_Module_V.2/pedestal_readout\" \n"
        << " Path format: /database/folder" << std::endl;
    streamlog_out(MESSAGE) << "\t\t -R --remap"
        << "\n\t\t\t Additionally remaps the pedestal data from the input files to the gear mapping. "
        << "For this function to work a valid channel mapping file should be provided as well. "
        << "The optional argument defines the DB folder where to store the data"
        << "Default: \"/LP1/T24_1/DESY/Grid_GEM_Module_V.2/pedestal_gear\" \n"
        << " Path format: /database/folder" << std::endl;
}

std::vector<std::ifstream*>
CreateInputStreams(std::string& inputfile)
{
    std::vector<std::ifstream*> result;
    if(( inputfile.length() > 6 ) && ( inputfile.substr(inputfile.length() - 6) == "_0.dat" ))
    {
        std::string Basename = inputfile.substr(0, inputfile.length() - 6);
        int counter = 0;
        std::ifstream* file = new std::ifstream(inputfile.c_str(), std::ifstream::binary);
        while(file->good())
        {
            result.push_back(file);
            std::ostringstream NextFilename;
            NextFilename << Basename << "_" << ++counter << ".dat";
            //streamlog_out(MESSAGE) << "Altro Raw Data file detected for this run: " << NextFilename.str() << std::endl;
            file = new std::ifstream(NextFilename.str().c_str(), std::ifstream::binary);
        }
        delete file;

        if(result.empty())
        {
            throw marlintpc::TAltroConverterCLException(std::string("Invalid input file detected: ") + inputfile);
        }
    }
    else
    {
        std::ifstream* file = new std::ifstream(inputfile.c_str(), std::ifstream::binary);
        if(!file->good())
        {
            delete file;
            throw marlintpc::TAltroConverterCLException(std::string("Invalid input file detected: ") + inputfile);
        }
        result.push_back(file);
    }
    return result;
}

std::string
GenerateOutFileName(std::string& inputfile)
{
    std::stringstream outfilename;

    if(( inputfile.length() > 6 ) && ( inputfile.substr(inputfile.length() - 6) == "_0.dat" ))
    {
        outfilename << inputfile.substr(0, inputfile.length() - 6) << ".slcio";
    }
    else
    {
        size_t lastdotpos = inputfile.find_last_of(".");
        outfilename << inputfile.substr(0, lastdotpos) << ".slcio";
    }
    return outfilename.str();
}

