/*
 * TAltroBlockFactory.hh
 *
 *  Created on: Jul 6, 2012
 *      Author: caiazza
 */

#ifndef TALTROBLOCKFACTORY_HH_
#define TALTROBLOCKFACTORY_HH_

#include "TIAltroBlockFactory.hh"

#include <utility>

namespace marlintpc
{

class TAltroBlockFactory : public TIAltroBlockFactory
{
public:
    /**
     * Standard and default constructor
     * @param DataVersion
     */
    TAltroBlockFactory(std::pair<int, int> DataVersion = std::make_pair(200,0));

    TAltroBlockFactory(const TAltroBlockFactory& other);

    virtual ~TAltroBlockFactory();

    TAltroBlockFactory& operator=(const TAltroBlockFactory& other);

    /**
     * Sets the data version for which to create the appropriate data blocks
     * @param a pair of integers representing the encoding format
     */
    virtual void SetDataVersion(std::pair<int, int> DataVersion) throw (TAltroConverterVersionException);

    /**
     * Creates a new TIAltroBlock object using the data from the input stream.
     * It assumes that the istream cursor is pointed to the start point of the block in the raw data
     * It will read the input stream until the end of the block which means that the read cursor will
     * be moved accordingly
     * @param input
     * @return a pointer to the newly created ALtroBlock
     */
    virtual TIAltroBlock* GetBlock(std::ifstream* input) throw (TAltroConverterException, std::exception);

    /**
     * Check if the input block is of type SOR
     * @param Block to be checked
     * @return Check result
     */
    virtual bool isSOR(TIAltroBlock* Block) const ;

    /**
     * Check if the input block is of type Event
     * @param Block to be checked
     * @return Check result
     */
    virtual bool isEvent(TIAltroBlock* Block) const ;

    /**
     * Check if the input block is of type EOR
     * @param Block to be checked
     * @return Check result
     */
    virtual bool isEOR(TIAltroBlock* Block) const ;

    /**
     * Check if the input block is of type BOF
     * @param Block to be checked
     * @return Check result
     */
    virtual bool isBOF(TIAltroBlock* Block) const ;

    /**
     * Check if the input block is of type EOF
     * @param Block to be checked
     * @return Check result
     */
    virtual bool isEOF(TIAltroBlock* Block) const ;

    /**
     * Check if the input block is of type POR
     * @param Block to be checked
     * @return Check result
     */
    virtual bool isPOR(TIAltroBlock* Block) const ;

    /**
     * Check if the input block is of type COR
     * @param Block to be checked
     * @return Check result
     */
    virtual bool isCOR(TIAltroBlock* Block) const ;

    /**
     * Check if the input block is an RCU block
     * @param Block
     * @return Check result
     */
    virtual bool isRCU(TIAltroBlock* Block) const;


private:
    std::pair<int, int> _DataVersion;

};

}

#endif /* TALTROBLOCKFACTORY_HH_ */
