/*
 * TAltroPedestalConverter.hh
 *
 *  Created on: Aug 17, 2012
 *      Author: caiazza
 */

#ifndef TALTROPEDESTALCONVERTER_HH_
#define TALTROPEDESTALCONVERTER_HH_

#include "TIPedestalConverter.hh"

#include <map>

namespace marlintpc
{
/**
 * Implementation of a class to convert the Altro Pedestal data to the MarlinTPC standard
 */
class TAltroPedestalConverter : public TIPedestalConverter
{
public:
    /**
     * Default constructor.
     */
    TAltroPedestalConverter();

    /**
     * Virtual destructor
     */
    virtual ~TAltroPedestalConverter();

    /**
     * Copy constructor
     * @param The other Converter object
     */
    TAltroPedestalConverter(const TAltroPedestalConverter& other);

    /**
     * Assignment operator
     * @param The object to copy
     * @return The result of the copy to allow concatenation
     */
    TAltroPedestalConverter& operator=(const TAltroPedestalConverter& other);

    /**
     * Function call operator to simplify the call of the convert function
     * @param inputfilenames A list of filenames of the file to use for input
     * @return The collection of pedestal objects obtained from the file
     */
    lcio::LCCollection* operator()(const std::vector<std::string>& inputfilenames);

    /**
     * Converts the data from the files listed to an LCCollection
     * @param inputfilenames A list of filenames of the file to use for input
     * @return The collection of pedestal objects obtained from the file
     */
    virtual lcio::LCCollection* Convert(const std::vector<std::string>& inputfilenames);

    /**
     * Instructs the class to remap to GearID the pedestal collection using the given channel mapping collection.
     * @param MappingColl The collection containing the channel mapping information
     */
    virtual void Remap(const lcio::LCCollection* ChanMappingColl);

    /**
     * Provides the necessary information to store the collection to an lccd database
     * @param Database Interface to the database
     * @param Start Start Timestamp
     * @param End End Timestamp
     * @param ColName Collection name for the database storage
     */
    virtual void ToDB(lccd::DBInterface& Database,
        const lccd::LCCDTimeStamp& Start,
        const lccd::LCCDTimeStamp& End,
        const std::string& Description = "TPC Pedestals") const;

    /**
     * Provides the necessary information to store the pedestal collection in an slcio file.
     * @param filename Names of the output file
     * @param ColName Name of the collection
     */
    virtual void ToFile(const std::string& filename, const std::string& ColName ="TPCPedestal") const ;

    /**
     * Retrieves the Pedestal collection after conversion
     * @return An LCIO Collection containing the pedestal data
     */
    virtual lcio::LCCollection* GetPedestalCollection() const;

private:
    /**
     * Stores the converted pedestals
     */
    lcio::LCCollection* _ConvertedPedestals;

    /**
     * Stores the channel mapping information for the remapping of the pedestals
     */
    std::map< std::pair<int, int>,std::pair<int, int> > _ChannelMapping;

    /**
     * Defines whether the user wants to remap the pedestal data or not
     */
    bool _doMapping;



};
}


#endif /* TALTROPEDESTALCONVERTER_HH_ */
