/*
 * TIAltroBlock.hh
 *
 *  Created on: Jul 6, 2012
 *      Author: caiazza
 */

#ifndef TIALTROBLOCK_HH_
#define TIALTROBLOCK_HH_

#include <stdint.h>
#include <vector>

//LCIO
#include "lcio.h"
#include "EVENT/LCRunHeader.h"

#include "AltroConverterExceptions.hh"

namespace marlintpc
{
/**
 * Interface defining the common behaviour of all the ALTRO blocks
 */
class TIAltroBlock
{
public:
    /**
     * Virtual destructor
     */
    virtual ~TIAltroBlock(){}

    /**
     * Get the block lenght in number of 32 bit words
     * @return number of 32 bit words in the block
     */
    virtual uint32_t GetLength() const = 0;

    /**
     * Verifies that the content of the block matches the expected content of the type of block we build.
     * @return True if the content of the block matches the expectation, False otherwise
     */
    virtual bool CheckIntegrity() const = 0;

    /**
     * Returns a constant reference to the content of the block
     * @return vector of 32 bit data words
     */
    virtual const std::vector<uint32_t>& Content() const = 0;

    /**
     * Returns a data word that identifies the block type
     * @return 32 bit data word identifying the block
     */
    virtual uint32_t GetIdentifier() const = 0;

    /**
     * Allows to set additional parameters to the RunHeader or from the information already in the Run Header.
     * @param RunHeader
     */
    virtual void SetAdditionalParameters(lcio::LCRunHeader* RunHeader) = 0;
};
} //end namespace


#endif /* TIALTROBLOCK_HH_ */
