/*
 * TIBOFBlock.hh
 *
 *  Created on: Jul 10, 2012
 *      Author: caiazza
 */

#ifndef TIBOFBLOCK_HH_
#define TIBOFBLOCK_HH_

#include "TIAltroBlock.hh"

//LCIO
#include "lcio.h"
#include "UTIL/LCTime.h"

namespace marlintpc
{
class TIBOFBlock : public TIAltroBlock
{
public:
    /**
     * Virtual destructor
     */
    virtual ~TIBOFBlock(){}

    /**
     * Timestamp of the pause
     * @return Timestamp of the pause
     */
    virtual lcio::LCTime GetTimestamp() const = 0;

    /**
     * Number of events collected at the pause
     * @return Number of events collected at the pause
     */
    virtual uint32_t GetNEvents() const = 0;

    /**
     * Get the file number counter
     * @return File index
     */
    virtual uint32_t GetFileNumber() const = 0;

    /**
     * Get run number
     * @return Run Number
     */
    virtual uint32_t GetRunNumber() const = 0;
}; // End interface class

/**
 * Implementation of the Beginning of file block class for DAQ version 4.2+
 */
class TBOFBlock_V42 : public TIBOFBlock
{
public:
    TBOFBlock_V42(std::vector<uint32_t>& Content);

    virtual ~TBOFBlock_V42();

    /**
     * Get the block lenght in number of 32 bit words
     * @return number of 32 bit words in the block
     */
    virtual uint32_t GetLength() const;

    /**
     * Verifies that the content of the block matches the expected content of the type of block we build.
     * @return True if the content of the block matches the expectation, False otherwise
     */
    virtual bool CheckIntegrity() const;

    /**
     * Returns a constant reference to the content of the block
     * @return vector of 32 bit data words
     */
    virtual const std::vector<uint32_t>& Content() const;

    /**
     * Returns a data word that identifies the block type
     * @return 32 bit data word identifying the block
     */
    virtual uint32_t GetIdentifier() const;

    /**
     * Timestamp of the pause
     * @return Timestamp of the pause
     */
    virtual lcio::LCTime GetTimestamp() const;

    /**
     * Number of events collected at the pause
     * @return Number of events collected at the pause
     */
    virtual uint32_t GetNEvents() const;

    /**
     * Get the file number counter
     * @return File index
     */
    virtual uint32_t GetFileNumber() const;

    /**
     * Get run number
     * @return Run Number
     */
    virtual uint32_t GetRunNumber() const;

    /**
     * Allows to set additional parameters to the RunHeader.
     * @param RunHeader
     */
    virtual void SetAdditionalParameters(lcio::LCRunHeader* RunHeader);

private:
    std::vector<uint32_t> _content;
}; // End BOF V4.2 class

}

#endif /* TIBOFBLOCK_HH_ */
