/*
 * TIMappingConverter.hh
 *
 *  Created on: Jul 3, 2012
 *      Author: caiazza
 */

#ifndef TIMAPPINGCONVERTER_HH_
#define TIMAPPINGCONVERTER_HH_

//STD
#include <string>

//LCIO
#include "lcio.h"
#include "EVENT/LCCollection.h"

//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"

//GEAR
#include "gear/GearMgr.h"

namespace marlintpc
{
/**
 * Interface for a class to convert the channel mapping information from an ASCII file to LCIO.
 * The channel mapping is the mapping of each hardware id to a gear id.
 * It assumes the mapping information are stored in a single ASCII file and the geometry information
 * are to be retrieved through GEAR
 */
class TIMappingConverter
{
public:
    /**
     * Empty virtual destructor to enable polymorphic behaviours
     */
    virtual ~TIMappingConverter(){}

    virtual lcio::LCCollection* Convert(std::string inputfilename, gear::GearMgr* gearMgr) = 0;

    /**
     * Stores the information of the Input collection to the lccd database
     * @param Database Interface to the database
     * @param Start Start Timestamp
     * @param End End Timestamp
     * @param ColName Collection name for the database storage
     */
    virtual void ToDB(lccd::DBInterface Database, lccd::LCCDTimeStamp Start, lccd::LCCDTimeStamp End, std::string ColName = "TPCChannelMapping") = 0;

    /**
     * Stores the information stored in the Input collection to a file.
     * @param filename Names of the output file
     * @param ColName Name of the collection
     */
    virtual void ToFile(std::string filename, std::string ColName = "TPCChannelMapping") = 0;


};

}


#endif /* TIMAPPINGCONVERTER_HH_ */
