/*
 * TIPedestalConverter.hh
 *
 *  Created on: Jul 3, 2012
 *      Author: caiazza
 */

#ifndef TIPEDESTALCONVERTER_HH_
#define TIPEDESTALCONVERTER_HH_

//STD
#include <vector>
#include <string>

//LCIO
#include "lcio.h"
#include "EVENT/LCCollection.h"

//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"
namespace marlintpc
{
/**
 * Interface class for an object that converts the pedestal data from a set of ASCII file to LCIO.
 * It assumes the user is using the ilcsoft infrastructure which is LCCD and LCIO.
 */
class TIPedestalConverter
{
public:
    /**
     * Virtual destructor to enable polymorphic behaviors
     */
    virtual ~TIPedestalConverter(){}

    /**
     * Converts the data from the files listed to an LCCollection
     * @param inputfilenames A list of filenames of the file to use for input
     * @return The collection of pedestal objects obtained from the file
     */
    virtual lcio::LCCollection* Convert(const std::vector<std::string>& inputfilenames) = 0;

    /**
     * Instructs the class to remap to GearID the pedestal collection using the given channel mapping collection.
     * @param MappingColl The collection containing the channel mapping information
     */
    virtual void Remap(const lcio::LCCollection* ChanMappingColl) = 0;

    /**
     * Provides the necessary information to store the collection to an lccd database
     * @param Database Interface to the database
     * @param Start Start Timestamp
     * @param End End Timestamp
     * @param ColName Collection name for the database storage
     */
    virtual void ToDB(lccd::DBInterface& Database,
        const lccd::LCCDTimeStamp& Start,
        const lccd::LCCDTimeStamp& End,
        const std::string& ColName) const = 0;

    /**
     * Provides the necessary information to store the pedestal collection in an slcio file.
     * @param filename Names of the output file
     * @param ColName Name of the collection
     */
    virtual void ToFile(const std::string& filename, const std::string& ColName) const = 0;

    /**
     * Retrieves the Pedestal collection after conversion
     * @return An LCIO Collection containing the pedestal data
     */
    virtual lcio::LCCollection* GetPedestalCollection() const = 0;

};
}


#endif /* TIPEDESTALCONVERTER_HH_ */
