/*
 * DataFormatDecoder_V1.cpp
 *
 *  Created on: Jul 11, 2012
 *      Author: caiazza
 */

#include "DataFormatDecoder.hh"
#include "IMPL/TrackerRawDataImpl.h"

namespace marlintpc
{
DataFormatDecoder_V1::DataFormatDecoder_V1(std::vector<uint32_t>::iterator& BeginData, uint32_t RCUID):
        _beginData(BeginData),
        _RCUID(RCUID)
{

}

DataFormatDecoder_V1::~DataFormatDecoder_V1()
{

}

void
DataFormatDecoder_V1::AddData(lcio::LCCollection* Collection) const
{
    std::vector<uint32_t>::const_iterator index = _beginData;

    //Cycling through the pulses.
    while(((*index) & (WORDID)) == 0x40000000)
    {
        uint32_t chId = (*index) & (CHANNELADDRESS);

        //Read from the data the number of altro words in the next data block and moves to the next element in the data vector
        uint32_t words10bit = (*index & (DWNUM)) >> 16;
        index++;

        //creates a vector to be filled with the 10bit data words of the altro data format and reserves enough space
        std::vector<short> sampleList;
        sampleList.reserve((words10bit));

        //Now I fill the sample list with the data words
        for (uint32_t i = 0; i < words10bit; i++)
        {
            int wordPos = i % 3;
            switch (wordPos)
            {
            case 0:
                sampleList.push_back(((*index) & DW3) >> 20);
                break;
            case 1:
                sampleList.push_back(((*index) & DW2) >> 10);
                break;
            case 2:
                sampleList.push_back((*index) & DW1);
                index++;
                break;
            }
        }
        if ((words10bit % 3) != 0)
            index++;

        //Now I will decode the different types of data words and I will fill them in the LCIO collection
        std::vector<short>::const_iterator it = sampleList.begin();
        while (it != sampleList.end())
        {
            //Create the Tracker raw data. It will be destroyed by the LCIO framework.
            IMPL::TrackerRawDataImpl* Pulse = new IMPL::TrackerRawDataImpl();
            //Fill it with the time-space coordinates of the adc sample
            Pulse->setCellID1(_RCUID);
            Pulse->setCellID0(chId);

            //store the adc values
            short sampleLength = *it;

            //The timestamp in the LCIO format is the time of the first sample while in the
            //AltroDataFormat is the time of the last of the samples. Thus we need to perform a conversion
            Pulse->setTime((*(it + 1)) - (sampleLength -1));

            std::vector<short> adcValues;
            adcValues.reserve(sampleLength - 2);

            //Moving the iterator to the beginning of the next block
            it += sampleLength;
            //retrieving the samples backwords
            for (int i = 1; i <= (sampleLength - 2); i++)
            {
                adcValues.push_back(*(it - i));
            }
            Pulse->setADCValues(adcValues);
            Collection->addElement(Pulse);
        }
    }
}

}


