/*
 * TAltroPedestalConverter.cpp
 *
 *  Created on: Aug 17, 2012
 *      Author: caiazza
 */

#include "TAltroPedestalConverter.hh"
#include "MarlinTPCExceptions.hh"

#include <iostream>
#include <fstream>
#include <sstream>
#include <stdio.h>
#include <cstdlib>
#include <ctype.h>


//TPCCONDDATA
#include "Pedestal.h"
#include "ADCChannelMapping.h"

//ilcutil
#include "streamlog/streamlog.h"

//LCIO
#include "IMPL/LCEventImpl.h"
#include "IMPL/LCCollectionVec.h"

namespace marlintpc
{
TAltroPedestalConverter::TAltroPedestalConverter():
    _ConvertedPedestals(0),
    _ChannelMapping(),
    _doMapping(false)
{

}

TAltroPedestalConverter::~TAltroPedestalConverter()
{
    delete _ConvertedPedestals;
    _ConvertedPedestals = 0;
}

TAltroPedestalConverter::TAltroPedestalConverter(const TAltroPedestalConverter& other):
    _ConvertedPedestals(0),
    _ChannelMapping(other._ChannelMapping),
    _doMapping(other._doMapping)
{

}

TAltroPedestalConverter&
TAltroPedestalConverter::operator=(const TAltroPedestalConverter& other)
{
    if(this != &other)
    {
        delete _ConvertedPedestals;
        _ConvertedPedestals = 0;
        _ChannelMapping = other._ChannelMapping;
        _doMapping = other._doMapping;
    }
    return *this;
}

lcio::LCCollection*
TAltroPedestalConverter::operator()(const std::vector<std::string>& inputfilenames)
{
    return (Convert(inputfilenames));
}

lcio::LCCollection*
TAltroPedestalConverter::Convert(const std::vector<std::string>& inputfilenames)
{
    // create the collection we want to store the data in
    _ConvertedPedestals = new lcio::LCCollectionVec( lcio::LCIO::LCGENERICOBJECT );

    int fileNum=1;
    std::vector< std::pair<int,int> > warningList;
    // loop over the input files

    std::vector<std::string>::const_iterator inFile = inputfilenames.begin();
    for( ; inFile != inputfilenames.end(); ++inFile)
    {
        std::filebuf infile;
        infile.open(inFile->c_str(),std::ios::in);
        std::istream inputstream(&infile);
        std::cout<<" Reading file #"<<fileNum<<" : "<<*inFile<<std::endl;
        fileNum++;

        int rcu=0;
        int linenumber = -1;
        // read all lines in the file
        while (!inputstream.eof() )
        {
            // read the next line from the file
            std::string line;
            std::getline(inputstream, line);
            if (inputstream.eof())   //eof is reached
            {
                if (!line.empty())
                {
                    std::cout << " Ignoring non-empty line at end of file: " << line << std::endl;
                }
                break;
            }

            // use the line as a stringstream so we can stream into the variables
            std::stringstream linestream(line);
            linenumber++;
            // check if the next line is a comment
            if (linestream.peek() == '#')
            {
                std::stringbuf name_buf;
                linestream.get( name_buf, ' ' );
                std::string name = name_buf.str();
                if ( name == "#RCUID" )
                {
                    linestream.get(name_buf);
                    std::string dump=name_buf.str();
                    size_t found;
                    found=dump.find_last_of(' ');
                    std::string rcustring = dump.substr(found+1);
                    std::stringstream rcustream(rcustring);
                    rcustream >> rcu;
                    std::cout<<" Found in File RCU # = "<<rcu<<std::endl;
                }
                else
                {
                    std::cout << "  DEBUG: skipping comment line " << linenumber << " : "  << line <<  std::endl;
                }
                continue;
            }
            int hw_channel, intPedestal, counter;
            float floatPedestal, pedestalWidth;

            // stream the data into the variables
            linestream >>  hw_channel >> intPedestal >> floatPedestal >> pedestalWidth >> counter;
            // std::cout << "DEBUG:  reading line " << linenumber << " : "<< line << std::std::endl;
            // test if everything went fine
            if (linestream.fail())
            {
                std::cout << "error reading line " << linenumber << " : "<< line << std::endl;
                continue;
            }

            tpcconddata::Pedestal* pedestalObject=NULL;
            if (!_doMapping)
            {
                pedestalObject = new  tpcconddata::Pedestal(hw_channel, rcu , floatPedestal , pedestalWidth);
                _ConvertedPedestals->addElement( pedestalObject );
            }
            else
            {
                std::map< std::pair<int, int>, std::pair<int, int> >::iterator result =
                    _ChannelMapping.find(std::make_pair(hw_channel, rcu));
                if ( result != _ChannelMapping.end() )
                {
                    int padID=((*result).second).first;
                    int moduleID=((*result).second).second;
                    //std::cout<<"PadID/ModuleID="<<padID<<"/"<<moduleID<<std::endl;
                    pedestalObject = new  tpcconddata::Pedestal(padID, moduleID , floatPedestal , pedestalWidth);
                    _ConvertedPedestals->addElement( pedestalObject );
                }
                else
                {
                    if ( intPedestal!=1023)
                        warningList.push_back(std::make_pair(hw_channel,rcu));
                }
            }
        }                            //  while (!inputstream.eof() )
        // close the input text file
        infile.close();
    }

    if (warningList.size() >0 )
    {
        std::cerr<<"WARNING: the following pedestal values could not be mapped, since there is no entry in the pad mapping:"<<std::endl;
        for (unsigned int i=0; i<warningList.size(); i++)
        {
            std::cerr<<"("<<warningList[i].first<<","<<warningList[i].second<<") ";
        }
        std::cerr<<std::endl;
    }

    return _ConvertedPedestals;
}

void
TAltroPedestalConverter::Remap(const lcio::LCCollection* ChanMappingColl)
{
    if(ChanMappingColl == 0)
    {
        _doMapping = false;
        _ChannelMapping.clear();
    }
    else
    {
        _ChannelMapping.clear();
        _doMapping = true;

        tpcconddata::ADCChannelMapping* Entry = 0;
        std::pair<  std::map< std::pair<int, int>,std::pair<int, int> >::iterator,bool> iInsertReturn;
        int nEntries = ChanMappingColl->getNumberOfElements();
        for (int iEntry = 0; iEntry < nEntries; ++iEntry)
        {
            Entry = new tpcconddata::ADCChannelMapping( dynamic_cast<LCGenericObject*>( ChanMappingColl->getElementAt( iEntry ) ) );
            //std::cout<<"Inserting in the channel map: hw=("<<Entry->getChannelID()<<","<<Entry->getReadoutGroup()<<") gear=("<<Entry->getPadID()<<","<<Entry->getModuleID()<<") "<<std::endl;
            iInsertReturn=_ChannelMapping.insert( std::make_pair(std::make_pair(Entry->getChannelID(), Entry->getReadoutGroup()),
                                                            std::make_pair(Entry->getPadID(), Entry->getModuleID())        )
                                           );
            if ( !iInsertReturn.second )
            {
                streamlog_out(ERROR) << "ChannelMapping: could not store ChannelMapping for hardwareID: ("<<Entry->getChannelID()<<","<<Entry->getReadoutGroup()<<") in the readoutChannelMapping map because there was already a value stored for this channel"<<std::endl;
            }
        }
    }
}

void
TAltroPedestalConverter::ToDB(lccd::DBInterface& Database,
    const lccd::LCCDTimeStamp& Start,
    const lccd::LCCDTimeStamp& End,
    const std::string& Description) const
{
    if(_ConvertedPedestals == 0)
    {
        std::string ClassName = "TAltroPedestalConverter";
        std::stringstream err;
        err << "No Pedestal data are yet converted to LCIO format. Unable to write it to DB" << std::endl;
        throw NullPointerException(ClassName, err.str());
    }

    Database.storeCollection(Start, End, _ConvertedPedestals, Description);
}

 void
 TAltroPedestalConverter::ToFile(const std::string& filename, const std::string& ColName) const
 {
     if(_ConvertedPedestals == 0)
     {
         std::string ClassName = "TAltroPedestalConverter";
         std::stringstream err;
         err << "No Pedestal data are yet converted to LCIO format. Unable to write it to File" << std::endl;
         throw NullPointerException(ClassName, err.str());
     }

     lcio::LCWriter* lcWrt = lcio::LCFactory::getInstance()->createLCWriter();
     lcWrt->open( filename, lcio::LCIO::WRITE_NEW ) ;
     lcio::LCEvent* evt = new lcio::LCEventImpl();
     evt->addCollection( _ConvertedPedestals , ColName ) ;
     lcWrt->writeEvent( evt ) ;
     lcWrt->close() ;

     //Necessary to avoid the collection to be deleted with the event
     evt->takeCollection(ColName);
     delete evt;
 }

lcio::LCCollection*
TAltroPedestalConverter::GetPedestalCollection() const
{
    return _ConvertedPedestals;
}

}


