/*
 * TBOFBlock_V42.cpp
 *
 *  Created on: Jul 10, 2012
 *      Author: caiazza
 */

#include "TIBOFBlock.hh"
#include <string>
#include <sstream>
#include <iostream>

namespace marlintpc
{
TBOFBlock_V42::TBOFBlock_V42(std::vector<uint32_t>& Content):
        _content(Content)
{
    if(!CheckIntegrity())
    {
        std::stringstream message;
        message << "Failed integrity check while analyzing the Beginning of file Block" << std::endl;
        message << "Dumping the block Content" << std::hex << std::endl;
        std::vector<uint32_t>::iterator itContent = _content.begin();
        for(; itContent != _content.end(); ++itContent)
        {
            message << *itContent << std::endl;
        }
        message << std::dec;
        throw TAltroConverterDecodeException("TBOFBlock_V42", message.str());
    }
}

TBOFBlock_V42::~TBOFBlock_V42()
{

}

uint32_t
TBOFBlock_V42::GetLength() const
{
    return _content.size();
}

bool
TBOFBlock_V42::CheckIntegrity() const
{
    bool result = true;
    result = result && (GetLength() == _content.at(0) + 1)
            && (GetLength() >= 9)
            && (_content.at(2) == 0x44444444);

    return result;
}

const std::vector<uint32_t>&
TBOFBlock_V42::Content() const
{
    return _content;
}

uint32_t
TBOFBlock_V42::GetIdentifier() const
{
    return _content[2];
}

lcio::LCTime
TBOFBlock_V42::GetTimestamp() const
{
    int day = _content[5] & 0x000000ff;
    int month = (_content[5] & 0x0000ff00) >> 8;
    int year = (_content[5] & 0xffff0000) >> 16;
    int second = _content[6] & 0x000000ff;
    int minute = (_content[6] & 0x0000ff00) >> 8;
    int hour = (_content[6] & 0xffff0000) >> 16;
    return UTIL::LCTime(year, month, day, hour, minute, second);
}

uint32_t
TBOFBlock_V42::GetNEvents() const
{
    return _content[8];
}

uint32_t
TBOFBlock_V42::GetFileNumber() const
{
    return _content[7];
}

uint32_t
TBOFBlock_V42::GetRunNumber() const
{
    return _content[4];
}

void
TBOFBlock_V42::SetAdditionalParameters(lcio::LCRunHeader* RunHeader)
{

}

}


