/*
 * TPORBlock_V42.cpp
 *
 *  Created on: Jul 10, 2012
 *      Author: caiazza
 */

#include "TIPORBlock.hh"
#include <string>
#include <sstream>
#include <iostream>

namespace marlintpc
{
TPORBlock_V42::TPORBlock_V42(std::vector<uint32_t>& Content):
        _content(Content)
{
    if(!CheckIntegrity())
    {
        std::stringstream message;
        message << "Failed integrity check while analyzing the Pause of run Block" << std::endl;
        message << "Dumping the block Content" << std::hex << std::endl;
        std::vector<uint32_t>::iterator itContent = _content.begin();
        for(; itContent != _content.end(); ++itContent)
        {
            message << *itContent << std::endl;
        }
        message << std::dec;
        throw TAltroConverterDecodeException("TPORBlock_V42", message.str());
    }
}

TPORBlock_V42::~TPORBlock_V42()
{

}

uint32_t
TPORBlock_V42::GetLength() const
{
    return _content.size();
}

bool
TPORBlock_V42::CheckIntegrity() const
{
    bool result = true;
    result = result && (GetLength() == _content.at(0) + 1)
            && (GetLength() >= 6)
            && (_content.at(2) == 0x11112222);

    return result;
}

const std::vector<uint32_t>&
TPORBlock_V42::Content() const
{
    return _content;
}

uint32_t
TPORBlock_V42::GetIdentifier() const
{
    return _content[2];
}

lcio::LCTime
TPORBlock_V42::GetPauseTime() const
{
    int day = _content[4] & 0x000000ff;
    int month = (_content[4] & 0x0000ff00) >> 8;
    int year = (_content[4] & 0xffff0000) >> 16;
    int second = _content[5] & 0x000000ff;
    int minute = (_content[5] & 0x0000ff00) >> 8;
    int hour = (_content[5] & 0xffff0000) >> 16;
    return UTIL::LCTime(year, month, day, hour, minute, second);
}

uint32_t
TPORBlock_V42::GetEventsAtPause() const
{
    return _content[3];
}

void
TPORBlock_V42::SetAdditionalParameters(lcio::LCRunHeader* RunHeader)
{

}

}


