/*
 * TSORBlock_V46.cpp
 *
 *  Created on: Jul 10, 2012
 *      Author: caiazza
 */

#include "TISORBlock.hh"
#include <map>
#include <string>
#include <sstream>

namespace marlintpc
{

TSORBlock_V46::TSORBlock_V46(std::vector<uint32_t>& Content):
        TSORBlock_V42(Content)
{
    if(!CheckIntegrity())
    {
        std::stringstream message;
        message << "Failed integrity check while analyzing the Start of Run Block" << std::endl;
        throw TAltroConverterDecodeException("TSORBlock_V46", message.str());
    }

    std::vector<uint32_t> BlockContent = TSORBlock_V42::Content();

    _runSetsExt._overthreshold = (BlockContent[8] & 0x0000000c) >> 2;

    _runSetsExt._threshold = (BlockContent[8] & 0x00003ff0) >> 4;

    _runSetsExt._postsamples = (BlockContent[8] & 0x0001c000) >> 14;

    _runSetsExt._presamples = (BlockContent[8] & 0x00060000) >> 17;

    _runSetsExt._pretrigger = (BlockContent[8] & 0x00780000) >> 19;

    _runSetsExt._triggerSource = TriggerSource((BlockContent[8] & 0x01800000) >> 23);
}

TSORBlock_V46::~TSORBlock_V46()
{

}

bool
TSORBlock_V46::CheckIntegrity() const
{
    //no additional special conditions
    return TSORBlock_V42::CheckIntegrity();
}

lcio::LCRunHeader*
TSORBlock_V46::CreateRunHeader() const
{
    lcio::LCRunHeader* runHeader = TSORBlock_V42::CreateRunHeader();
    std::map<TriggerSource, std::string> TriggerSourceDescription;
    TriggerSourceDescription[DATA] = "Data Driven and Parallel Port";
    TriggerSourceDescription[DBOX] = "DBOX";
    TriggerSourceDescription[DATA_DBOX] = "Data Driven with DBOX";

    runHeader->parameters().setValue("Samples above ZS threshold", _runSetsExt._overthreshold);
    runHeader->parameters().setValue("Zero Suppression Threshold", _runSetsExt._threshold);
    runHeader->parameters().setValue("Number of postsample", _runSetsExt._postsamples);
    runHeader->parameters().setValue("Number of presample", _runSetsExt._presamples);
    runHeader->parameters().setValue("Number of pretrigger samples", _runSetsExt._pretrigger);
    runHeader->parameters().setValue("Trigger Source", _runSetsExt._triggerSource);
    runHeader->parameters().setValue("Trigger Source Description", TriggerSourceDescription[_runSetsExt._triggerSource]);

    return runHeader;
}

}//end namespace


