/*
 * EORBlock.cpp
 *
 *  Created on: Jul 25, 2011
 *      Author: caiazza
 */

#include "EORBlock.hh"
#include "AltroBlocksLib.h"

namespace A2Lconverter
{
EORBlock::EORBlock(std::ifstream * const inputFile)
        : AltroBlock(inputFile)
{

}
EORBlock::EORBlock(std::ifstream* const inputFile, int start_address,
        uint32_t length)
        : AltroBlock(inputFile, start_address, length)
{

}

EORBlock::EORBlock(const AltroBlock & theMotherBlock)
        : AltroBlock(theMotherBlock)
{

    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_EOR;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the EOR block doesn't appear to be a valid EOR block");
    }
}

EORBlock::EORBlock(const std::vector<uint32_t> & refContent)
        : AltroBlock(refContent)
{

}

void
EORBlock::Decode()
{
    try
    {
        if (content.size() == 0)
        {
            inputFile->clear();
            inputFile->exceptions(std::ifstream::badbit);
            inputFile->seekg(start, std::ios::beg);
            length = ReadWord32() + 1;
            end = start + length * WORD32;
            inputFile->seekg(start, std::ios::beg);
            FillContent(length);
        }
        else
        {
            length = content.size();
            end = start + length * WORD32;
        }
        if (content[0] != content[1] + 1)
            throw AltroBlockException(
                    "The total length doesn't match the header length +1");
        if (content[2] != AltroBlock::BLOCK_EOR)
            throw AltroBlockException(
                    "The block identification doesn't match with BLOCK_EOR");
        id = AltroBlock::BlockIds(content[2]);
        totEvents = content[3];
        int day = content[4] & 0x000000ff;
        int month = (content[4] & 0x0000ff00) >> 8;
        int year = (content[4] & 0xffff0000) >> 16;
        int second = content[5] & 0x000000ff;
        int minute = (content[5] & 0x0000ff00) >> 8;
        int hour = (content[5] & 0xffff0000) >> 16;
        timestamp = UTIL::LCTime(year, month, day, hour, minute, second);
    }
    catch(std::ifstream::failure &f)
    {
        std::ostringstream err;
        err
                << "ERROR: Exception caught while decoding the EOR block. The file ended before it was supposed to"
                << std::endl;
        err << f.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
    catch (std::exception &e)
    {
        std::ostringstream err;
        err << "ERROR: Exception caught while decoding the EOR block. "
                << std::endl;
        err << e.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
}

std::string
EORBlock::PrintSummary() const
{
    std::ostringstream oss;
    oss << std::dec << "End of run summary" << std::endl << std::endl;
    oss << "Run end: " << timestamp.getDateString() << std::endl;
    oss << "Number of events recorded: " << totEvents << std::endl << std::endl;

    return oss.str();
}

bool
EORBlock::CheckIntegrity() const
{
    if (AltroBlock::CheckIntegrity())
    {
        if (content.size() == 0)
        {
            inputFile->seekg(start + 2 * WORD32, std::ios::beg);
            return (EORBlock::ReadWord32() == AltroBlock::BLOCK_EOR);
        }
        else
        {
            return (content[2] == AltroBlock::BLOCK_EOR);
        }
    }
    return false;
}

uint32_t
EORBlock::GetTotEvents() const
{
    return totEvents;
}




}


