/*
 * EventBlock.cpp
 *
 *  Created on: Jul 26, 2011
 *      Author: caiazza
 */

#include "EventBlock.hh"
#include "AltroBlocksLib.h"

namespace A2Lconverter
{
uint32_t EventBlock::eventsRead = 0;
EVENT::long64 EventBlock::tstampSkip = 0;
EVENT::long64 EventBlock::baseTStamp = 0;

EventBlock::EventBlock(std::ifstream * const inputFile)
        : AltroBlock(inputFile)
{

}
EventBlock::EventBlock(std::ifstream* const inputFile, int start_address,
        uint32_t length)
        : AltroBlock(inputFile, start_address, length)
{

}

EventBlock::EventBlock(const AltroBlock & theMotherBlock)
        : AltroBlock(theMotherBlock)
{

    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_EVENT;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the EVENT block doesn't appear to be a valid EVENT block");
    }
}

EventBlock::EventBlock(const std::vector<uint32_t> & refContent)
        : AltroBlock(refContent)
{

}

void
EventBlock::Decode()
{
    try
    {
        AltroBlock::Decode();
        if (content[2] != AltroBlock::BLOCK_EVENT)
            throw AltroBlockException(
                    "The block identification doesn't match with BLOCK_EVENT");
        id = AltroBlock::BlockIds(content[2]);
        eventNr = content[3];
        if(eventNr%100 == 0)
            std::cout << "DEBUG: Decoding event block: " << eventNr << std::endl;
        if (eventNr != (eventsRead + 1))
        {
            std::cout << "WARNING: This event seems to be out of sequence."
                    << std::endl << "The total number of event read is "
                    << eventsRead << " and this is the event number " << eventNr
                    << std::endl;
        }
        eventsRead++;
        DBOXEVTN = content[4];
        DBOXTIME = content [5];
        TLUEVTN = content[6];
        if(tstampSkip >= 0)
        {
            timestamp = UTIL::LCTime(baseTStamp + tstampSkip * static_cast<EVENT::long64>(eventsRead + 1));
        }
        else
        {
            timestamp = UTIL::LCTime(static_cast<EVENT::long64>(DBOXTIME));
        }

        std::vector<uint32_t>::const_iterator nextBlockStart = content.begin() + 7;
        while(nextBlockStart != content.end())
        {
            RCUBlock DataBlock = RCUBlock(nextBlockStart);
            DataBlock.Decode();
            Data.push_back(DataBlock);
            nextBlockStart += DataBlock.GetBlockLength();
        }

    }
    catch (std::ifstream::failure &f)
    {
        std::ostringstream err;
        err
                << "ERROR: Exception caught while decoding the Event block. The file ended before it was supposed to"
                << std::endl;
        err << f.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
    catch (std::exception &e)
    {
        std::ostringstream err;
        err << "ERROR: Exception caught while decoding the Event block. "
                << std::endl;
        err << e.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
}

void
EventBlock::Write2LCIO(IO::LCWriter* lcwriter) const
{
    lcio::LCEventImpl * lcevent = new lcio::LCEventImpl;

    lcevent->setEventNumber(eventNr );
    lcevent->setRunNumber( SORBlock::GetRunNumber() );
    lcevent->setTimeStamp(timestamp.timeStamp());

    for(std::vector<RCUBlock>::const_iterator it = Data.begin(); it != Data.end(); it++)
    {
        it->AddCollections(lcevent);
    }

    lcwriter->writeEvent(lcevent);

    delete lcevent;

}

std::string
EventBlock::PrintSummary() const
{
    std::ostringstream oss;
    oss << std::dec << "Event Number: " << eventNr << std::endl << std::endl;
    return oss.str();
}

bool
EventBlock::CheckIntegrity() const
{
    if (AltroBlock::CheckIntegrity())
    {
        if (content.size() == 0)
        {
            inputFile->seekg(start + 2 * WORD32, std::ios::beg);
            return (ReadWord32() == AltroBlock::BLOCK_EVENT);
        }
        else
        {
            return (content[2] == AltroBlock::BLOCK_EVENT);
        }
    }
    return false;
}

}

