#ifndef FIELD_H
#define FIELD_H 1

// stl
#include <string>

// CLHEP
#include "CLHEP/Matrix/Vector.h"
#include "CLHEP/Vector/ThreeVector.h"

namespace EVENT {

	class LCCollection;
}

namespace marlintpc {

/** Field : A base interface to an electric or magnetic field
 *
 *    @author J. Abernathy (UVic)<br>
 */
class Field {

public:
	/** constructor
	 */
	Field( std::string field_typename );

	/** destructor
	 */
	virtual ~Field();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void ) = 0;

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * ) = 0;

	/** gets the field value at a specified point (in carthesian coordinates)
	 * @deprecated replaced by get_field_value(const CLHEP::Hep3Vector&)}
	 */
	CLHEP::HepVector get_field_value( const CLHEP::HepVector & );

	/** gets the field value at a specified point
	 */
	virtual CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & ) = 0;

	/** gets the field error. Assumes that the error is symmetric and independent of position
	 */
	virtual CLHEP::Hep3Vector get_field_error( );

	/** returns the typename of the field
	 */
	std::string get_typename();

protected:

	CLHEP::Hep3Vector _field_error;

private:

	std::string _field_typename;
};

} // namespace marlintpc

#endif // FIELD_H
