#ifndef MAGNETICFIELD_H
#define MAGNETICFIELD_H 1



// MarlinTPC
#include "Field.h"

// CLHEP
#include "CLHEP/Vector/ThreeVector.h"
#include "CLHEP/Vector/EulerAngles.h"


namespace marlintpc {

/** MagneticField : A base interface to a magnetic field
 *
 *    @author Christian Grefe <br>
 */
class MagneticField: public Field {

public:
	/** constructor
	 */
	MagneticField( std::string field_typename );

	/** destructor
	 */
	virtual ~MagneticField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void ) = 0;

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * ) = 0;

	/** gets the field value at a specified point
	 */
	virtual CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & );

	/** gets the field error. Assumes that the error is symmetric and independent of position
	 */
	virtual CLHEP::Hep3Vector get_field_error( );

	/** sets the relative position of the origin of the field map (i.e. center of the coil) to the general coordinate system.
	 * The translation is applied after the rotation, so when the axis of original system and coil system are parallel.
	 */
	void set_translation ( const CLHEP::Hep3Vector & );

	/** sets the orientation of the TPC with respect to the coil axis
	 */
	void set_rotation ( const double &, const double & );

	/** sets the scale factor for the magnetic field, i.e. higher current than during mapping
	 */
	void set_scale_factor ( const double & );


protected:

	/** gets the field value at a point given in coil coordinates
	 */
	virtual CLHEP::Hep3Vector get_field_value_coil_coordinates( const CLHEP::Hep3Vector & ) = 0;

private:

	CLHEP::Hep3Vector _translation;
	double _x_rotation;
	double _y_rotation;
	double _scale_factor;
};

} // namespace marlintpc

#endif // MAGNETICFIELD_H
