#ifdef USE_CGA

// stl
#include <string>
#include <cassert>

// Marlin
#include <marlin/Global.h>

// GEAR
#include <gear/BField.h>
#include <gearimpl/Vector3D.h>

// Mokka
#include <CGADefs.h>

// LCIO
#include <EVENT/LCCollection.h>
#include <EVENT/LCGenericObject.h>
#include <EVENT/LCRunHeader.h>

// marlintpc
#include "CGAMagneticField.h"

using namespace CLHEP;
using namespace EVENT;
using namespace std;
using namespace gear;

namespace marlintpc {

/** static variables
 */
bool CGAMagneticField::_cga_initialized = false;

/** constructor
 */
CGAMagneticField::CGAMagneticField ( LCRunHeader *run_header ):
	Field("CGAMagneticField") {

	if( !_cga_initialized ) {

		CGAInit( run_header->parameters().getStringVal("MOKKA_SteeringFile").c_str(), "", "", "", "", "" );

		_cga_initialized = true;
	}
}


/** destructor
 */
CGAMagneticField::~CGAMagneticField() {}


/** returns a new field... similar to how the Marlin processor registration works
 This should never be called for a CGA field. Construction for those fields should be done in the run header for now
*/
Field *CGAMagneticField::new_field( void ) { return 0; }


/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
*/
bool CGAMagneticField::construct( LCCollection *collection ) {

	return true;
}


/** gets the field value at a specified point
*/
Hep3Vector CGAMagneticField::get_field_value( const Hep3Vector &position ) {

	double the_exact_same_position[3];
	the_exact_same_position[0] = position[0];
	the_exact_same_position[1] = position[1];
	the_exact_same_position[2] = position[2];

	// calculate B and E (E is constant for now)
	const vector< double > B = CGAGetB( the_exact_same_position );

	Hep3Vector field_vector ( B[0], B[1], B[2] );

	return field_vector;
}

} // namespace marlintpc

#endif // USE_CGA
