#ifndef LC_OBJECT_COPIER_H
#define LC_OBJECT_COPIER_H 1

#include <IMPL/TrackerPulseImpl.h>
#include <IMPL/TrackImpl.h>
#include <IMPL/TrackerDataImpl.h>
#include <IMPL/TrackerRawDataImpl.h>
#include <IMPL/MCParticleImpl.h>
#include <IMPL/SimTrackerHitImpl.h>

// this is a helper class for lcio objects, so I put it into the lcio namespace
namespace lcio
{

/** LCObjectCopier is a helper class which gives a copy of a class without the
    readOnly flag set, which is the case if you copy a object from a file using
    the copy constructor. It always returns an IMPL of the class, as the abstract
    EVENT classes cannot be instantiated. But you want an IMPL anyhow since you want to
    modify it. Otherwise you would not care about readOnly and thus not need this class at all.<br>
    ATTENTION:
    \li Has to be implemented for each class that has to be copied!
    \li If a class is extended, the code has to be adapted! Otherwise only part
        of the data is copied.<br>
        You chould state the LCIO version for each class.

    To implement for a new lcio-class, just overload the static clone() function.
*/

class LCObjectCopier
{

    private:
        /** A private, empty constructor.
            You cannot create an instance. All functions are static.
        */
        LCObjectCopier() {};


    public:
        /** Copy function for the TrackerPulse. LCIO v01-08<br>
        Attention, does only copy the pointer of TrackerData
        This is no clone/ deep copy function.
        */
        static IMPL::TrackerPulseImpl * copy(const EVENT::TrackerPulse * const inPulse);

        /** Copy function for the Track. LCIO v01-11<br>
         *  The tracker hits are alway a subcollection, so it's save to just
         *  copy the pointers.
        */
        static IMPL::TrackImpl * copy(const EVENT::Track * const track);

        /** Copy function for the Tracker without the hits (hits cannot be removed from
         *  a track, so this is useful to preserve the track parameters).
        */
        static IMPL::TrackImpl * copyTrackWithoutHits(const EVENT::Track * const track);

        /**
         * Copy function for the TrackerData.
         */
        static IMPL::TrackerDataImpl* copy(const EVENT::TrackerData* const inData);

        /**
         * Copy function for the TrackerRawData.
         */
        static IMPL::TrackerRawDataImpl* copy(const EVENT::TrackerRawData* const inRawData);

        /**
         * Copy function for the MCParticle.
         */
        static IMPL::MCParticleImpl* copy(const EVENT::MCParticle* const inParticle);

        /**
         * Copy function for SimTrackerHit.
         */
        static IMPL::SimTrackerHitImpl* copy(const EVENT::SimTrackerHit* const inHit);
};
}

#endif //OBJECT_COPIER_H
