/*
 * ExtendedAltroParameters.cc
 *
 *  Created on: Jul 13, 2012
 *      Author: caiazza
 */

#include <iostream>
#include <string>
#include <sstream>

#include "ExtendedAltroParameters.h"
#include "MarlinTPCExceptions.hh"

namespace tpcconddata
{

ExtendedAltroParameters::ExtendedAltroParameters(ExtendedAltroData& Data)
{
    SetData(Data);
}

ExtendedAltroParameters::ExtendedAltroParameters(EVENT::LCObject* obj):
        UTIL::LCFixedObject<EXTENDEDALTROPARSNINT, EXTENDEDALTROPARSNFLOAT, EXTENDEDALTROPARSNDOUBLE>(obj)
{

}

ExtendedAltroParameters::~ExtendedAltroParameters()
{

}

void
ExtendedAltroParameters::SetData(ExtendedAltroData& Data)
{
    SetPreampEnable(Data.PreampEnable);
    SetGainSettings(Data.Gain);
    SetShapingTime(Data.ShapingTime);
    SetShutdown(Data.Shutdown);
    SetZSThreshold(Data.ZSThreshold);
    SetZSGlitchFilter(Data.ZSGlitchFilter);
    SetPreSamples(Data.PreSamples);
    SetPostSamples(Data.PostSamples);
    SetPreTriggerSamples(Data.PreTriggerSamples);
    SetRCUAddress(Data.RCUAddress);
    SetRCUFWVersion(Data.RCUFWVersion);
    SetRCUActiveFECA(Data.RCUActiveFECA);
    SetRCUActiveFECB(Data.RCUActiveFECB);
    SetRCUConfigRegister1(Data.RCUConfigRegister1);
    SetRCUConfigRegister2(Data.RCUConfigRegister2);
    SetRCUStatusErrors(Data.RCUStatusErrors);
}

ExtendedAltroParameters::ExtendedAltroData
ExtendedAltroParameters::GetData() const
{
    ExtendedAltroData result;
    result.PreampEnable = GetPreampEnable();
    result.Gain = GetGainSettings();
    result.ShapingTime = GetShapingTime();
    result.Shutdown = GetShutdown();
    result.ZSThreshold = GetZSThreshold();
    result.ZSGlitchFilter = GetZSGlitchFilter();
    result.PreSamples = GetPreSamples();
    result.PostSamples = GetPostSamples();
    result.PreTriggerSamples = GetPreTriggerSamples();
    result.RCUAddress = GetRCUAddress();
    result.RCUFWVersion = GetRCUFWVersion();
    result.RCUActiveFECA = GetRCUActiveFECA();
    result.RCUActiveFECB = GetRCUActiveFECB();
    result.RCUConfigRegister1 = GetRCUConfigRegister1();
    result.RCUConfigRegister2 = GetRCUConfigRegister2();
    result.RCUStatusErrors = GetRCUStatusErrors();

    return result;
}

void
ExtendedAltroParameters::SetPreampEnable(bool& Enable)
{
    obj()->setIntVal(0, Enable);
}

void
ExtendedAltroParameters::SetGainSettings(GainSettings& Gain)
{
    obj()->setIntVal(1, Gain);
}

void
ExtendedAltroParameters::SetShapingTime(ShapingSettings& ShapingTime)
{
    obj()->setIntVal(2, ShapingTime);
}

void
ExtendedAltroParameters::SetShutdown(bool& Shutdown)
{
    obj()->setIntVal(3, Shutdown);
}

void
ExtendedAltroParameters::SetZSThreshold(int& Threshold)
{
    if(Threshold > 1024)
    {
        std::stringstream message;
        message << "The Zero Suppression Threshold for the ALTRO electronics has to be between "
                << "0 and 1023 and is being set at " << Threshold << std::endl;
        throw marlintpc::InvalidDataException("ExtendedAltroParameters", message.str());
    }

    obj()->setIntVal(4, Threshold);
}

void
ExtendedAltroParameters::SetZSGlitchFilter(int &MinSamples)
{
    if( (MinSamples < 0) | (MinSamples > 3))
    {
        std::stringstream message;
        message << "The Glith Filter minimum samples for the ALTRO electronics has to be between "
                << "0 and 3 and is being set at " << MinSamples << std::endl;
        throw marlintpc::InvalidDataException("ExtendedAltroParameters", message.str());
    }

    obj()->setIntVal(5, MinSamples);
}

void
ExtendedAltroParameters::SetPreSamples(int& PreSamples)
{
    if( (PreSamples < 0) | (PreSamples > 3) )
    {
        std::stringstream message;
        message << "The Number of PreSamples for the ALTRO electronics has to be between "
                << "0 and 3 and is being set at " << PreSamples << std::endl;
        throw marlintpc::InvalidDataException("ExtendedAltroParameters", message.str());
    }

    obj()->setIntVal(6, PreSamples);
}

void
ExtendedAltroParameters::SetPostSamples(int& PostSamples)
{
    if( (PostSamples < 0) | (PostSamples > 7) )
    {
        std::stringstream message;
        message << "The Number of PostSamples for the ALTRO electronics has to be between "
                << "0 and 7 and is being set at " << PostSamples << std::endl;
        throw marlintpc::InvalidDataException("ExtendedAltroParameters", message.str());
    }

    obj()->setIntVal(7, PostSamples);
}

void
ExtendedAltroParameters::SetPreTriggerSamples(int& PreTriggerSamples)
{
    if( (PreTriggerSamples < 0) | (PreTriggerSamples > 15) )
    {
        std::stringstream message;
        message << "The Number of PostSamples for the ALTRO electronics has to be between "
                << "0 and 15 and is being set at " << PreTriggerSamples << std::endl;
        throw marlintpc::InvalidDataException("ExtendedAltroParameters", message.str());
    }

    obj()->setIntVal(8, PreTriggerSamples);
}

void
ExtendedAltroParameters::SetRCUAddress(int& RCUAddress)
{
    obj()->setIntVal(9, RCUAddress);
}

void
ExtendedAltroParameters::SetRCUFWVersion(int& RCUFWVersion)
{
    obj()->setIntVal(10, RCUFWVersion);
}

void
ExtendedAltroParameters::SetRCUActiveFECA(int& RCUActiveFECA)
{
    obj()->setIntVal(11, RCUActiveFECA);
}

void
ExtendedAltroParameters::SetRCUActiveFECB(int& RCUActiveFECB)
{
    obj()->setIntVal(12, RCUActiveFECB);
}

void
ExtendedAltroParameters::SetRCUConfigRegister1(int& RCUConfigRegister1)
{
    obj()->setIntVal(13, RCUConfigRegister1);
}

void
ExtendedAltroParameters::SetRCUConfigRegister2(int& RCUConfigRegister2)
{
    obj()->setIntVal(14, RCUConfigRegister2);
}

void
ExtendedAltroParameters::SetRCUStatusErrors(int& RCUStatusErrors)
{
    obj()->setIntVal(15, RCUStatusErrors);
}

bool
ExtendedAltroParameters::GetPreampEnable() const
{
    return static_cast<bool>(getIntVal(0));
}

ExtendedAltroParameters::GainSettings
ExtendedAltroParameters::GetGainSettings() const
{
    return ExtendedAltroParameters::GainSettings(getIntVal(1));
}

ExtendedAltroParameters::ShapingSettings
ExtendedAltroParameters::GetShapingTime() const
{
    return ExtendedAltroParameters::ShapingSettings(getIntVal(2));
}

bool
ExtendedAltroParameters::GetShutdown() const
{
    return static_cast<bool>(getIntVal(3));
}

int
ExtendedAltroParameters::GetZSThreshold() const
{
    return getIntVal(4);
}

int
ExtendedAltroParameters::GetZSGlitchFilter() const
{
    return getIntVal(5);
}

int
ExtendedAltroParameters::GetPreSamples() const
{
    return getIntVal(6);
}

int
ExtendedAltroParameters::GetPostSamples() const
{
    return getIntVal(7);
}

int
ExtendedAltroParameters::GetPreTriggerSamples() const
{
    return getIntVal(8);
}

int
ExtendedAltroParameters::GetRCUAddress() const
{
    return getIntVal(9);
}

int
ExtendedAltroParameters::GetRCUFWVersion() const
{
    return getIntVal(10);
}

int
ExtendedAltroParameters::GetRCUActiveFECA() const
{
    return getIntVal(11);
}

int
ExtendedAltroParameters::GetRCUActiveFECB() const
{
    return getIntVal(12);
}

int
ExtendedAltroParameters::GetRCUConfigRegister1() const
{
    return getIntVal(13);
}

int
ExtendedAltroParameters::GetRCUConfigRegister2() const
{
    return getIntVal(14);
}

int
ExtendedAltroParameters::GetRCUStatusErrors() const
{
    return getIntVal(15);
}

std::string
ExtendedAltroParameters::getRevision()
{
    return std::string("$Rev: 3117 $");
}

std::string
ExtendedAltroParameters::getDefaultColName()
{
    return std::string("ExtendedAltroElectronicsParameters");
}

void
ExtendedAltroParameters::print(  std::ostream& os ) const
{
    std::streamsize PrecisionBefore = os.precision();
    std::streamsize WidthBefore = os.width();
    os.precision(4);
    os.setf(std::ios_base::fixed);
    os << "Extended Electronics Parameters: " << std::endl
            << "PreampEnable: " << GetPreampEnable() << "\t Gain: " << static_cast<int>(GetGainSettings()) <<std::endl
            << "Shaping Time: " << static_cast<int>(GetShapingTime()) << "\t Shutdown: " << GetShutdown() << std::endl
            << "Zero Suppression Threshold: " << GetZSThreshold() << "\t PreSamples: " << GetPreSamples() << std::endl
            << "PostSamples: " << GetPostSamples() << "\t PreTrigger Samples: " << GetPreTriggerSamples() << std::endl
            << "RCU Address: " << std::hex << GetRCUAddress() << std::dec << "\t RCU Firmware Version: " << GetRCUFWVersion() << std::endl
            << "Active FEC A: " <<std::hex << GetRCUActiveFECA() << "\t Active FEC B: " << GetRCUActiveFECB() << std::endl
            << "RCU Configuration Reg 1: " << GetRCUConfigRegister1() << "\t RCU Configuration Reg 2: " << GetRCUConfigRegister2() << std::endl
            << "RCU Status/Errors: " << GetRCUStatusErrors() << std::dec << std::endl;

    os.precision( PrecisionBefore );
    os.width( WidthBefore );
}

const std::string
ExtendedAltroParameters::getTypeName() const
{
    return std::string("ExtendedAltroParameters");
}

const std::string
ExtendedAltroParameters::getDataDescription() const
{
    std::string Description = "i:PreampEnable,Gain,ShapingTime,Shutdown,ZSThreshold,PreSamples,";
    Description += "PostSamples,PreTriggerSamples,RCUAddress,RCUFWVersion,RCUActiveFECA,";
    Description += "RCUActiveFECB,RCUConfigRegister1,RCUConfigRegister2,RCUStatusErrors";
    return Description;
}

std::ostream &operator<<(std::ostream &os, const ExtendedAltroParameters &p)
{
    p.print(os);
    return os;
}

bool operator==(const ExtendedAltroParameters &first, const ExtendedAltroParameters &second)
{
    bool result = first.GetPreampEnable() == second.GetPreampEnable() &&
            first.GetGainSettings() == second.GetGainSettings() &&
            first.GetShapingTime() == second.GetShapingTime() &&
            first.GetShutdown() == second.GetShutdown() &&
            first.GetZSThreshold() == second.GetZSThreshold() &&
            first.GetPreSamples() == second.GetPreSamples() &&
            first.GetPostSamples() == second.GetPostSamples() &&
            first.GetPreTriggerSamples() == second.GetPreTriggerSamples() &&
            first.GetRCUAddress() == second.GetRCUAddress() &&
            first.GetRCUFWVersion() == second.GetRCUFWVersion() &&
            first.GetRCUActiveFECA() == second.GetRCUActiveFECA() &&
            first.GetRCUActiveFECB() == second.GetRCUActiveFECB() &&
            first.GetRCUConfigRegister1() == second.GetRCUConfigRegister1() &&
            first.GetRCUConfigRegister2() == second.GetRCUConfigRegister2() &&
            first.GetRCUStatusErrors() == second.GetRCUStatusErrors();
    return result;
}

bool operator!=(const ExtendedAltroParameters &first, const ExtendedAltroParameters &second)
{
    return !(first == second);
}



}//end namespace


