#include "TimePixMapListener.h"
#include "TimePixPixelMode.h"\

//LCIO
#include "UTIL/LCTOOLS.h"

namespace tpcconddata{

TimePixMapListener::TimePixModeMapListener::TimePixModeMapListener
(TimePixMapListener *mapListener) :
  _mapListener(mapListener) {}


void TimePixMapListener::TimePixModeMapListener::conditionsChanged( LCCollection* col ) {
  //this function is called by the ConditionsProcessor 
  //each time a registerd collection changes

  _mapListener->modeChanged(col);

}

TimePixMapListener::TimePixStatusMapListener::TimePixStatusMapListener
(TimePixMapListener *mapListener) :
  _mapListener(mapListener) {}


void TimePixMapListener::TimePixStatusMapListener::conditionsChanged( LCCollection* col ) {
  //this function is called by the ConditionsProcessor 
  //each time a registerd collection changes

  _mapListener->statusChanged(col);

}

TimePixMapListener::TimePixMapListener() :
  _modeMapListener( this ) , _statusMapListener( this ) {}


void TimePixMapListener::modeChanged( LCCollection* col ) {
    
  //  LCTOOLS::printLCGenericObjects (col);
  
  for ( int i = 0; i < col->getNumberOfElements(); i++ ) {
 
    TimePixPixelMode *pm = 
      new TimePixPixelMode(dynamic_cast<LCGenericObject*>(col->getElementAt( i )));
    
    if ( _timePixMap.count( pm->getPixelKey() ) == 0 ) {
      //there is no elemet for this key yet, hence we will create one

      _timePixMap[ pm->getPixelKey() ] = new TimePixPixelMode( pm->getChipID(),
							       pm->getPixelID(),
							       pm->getPixelMode(),
							       pm->getPixelStatus() );

    }
    else{
      //this element already exists, all we have to do is to reset the pixelMode

      (_timePixMap[ pm->getPixelKey() ])->setPixelMode( pm->getPixelMode() );
	
    } //else
    

    delete pm;


  } //for ( int i = 0; i < col->getNumberOfElements(); i++ )



  std::cout<<" TimePixMapListener::modeChanged: "
	   <<"_timePixMap.size() = "<<_timePixMap.size()<<std::endl;
  
} // void modeChanged( LCCollection* col )


void TimePixMapListener::statusChanged( LCCollection* col ) {

  //  LCTOOLS::printLCGenericObjects (col);

  int newstatus = 0;
  int newintactstatus = 0;
  int newdead = 0;
  int newnoisy = 0;

  if ( !_timePixMap.empty() ) {
    //modeChanged had already been called once

    bool intact = true;

    for ( TimePixMap::iterator j = _timePixMap.begin(); j != _timePixMap.end(); j++ ) {
      
      for ( int i = 0; i < col->getNumberOfElements(); i++ ) {
	
	TimePixPixelMode *pm = 
	  new TimePixPixelMode(dynamic_cast<LCGenericObject*>(col->getElementAt( i )));

	if ( (*j).first == pm->getPixelKey() ) {
	  //the object in the map corresponds to the same pixel as pm
	  //change its status to the status of pm
	  
	  (*j).second->setPixelStatus( pm->getPixelStatus() );

	  if ( pm->getPixelStatus() == TimePixPixelMode::NOISY ) newnoisy++;
	  if ( pm->getPixelStatus() == TimePixPixelMode::DEAD  ) newdead++;

	  intact = false;
	  newstatus++;
	  
	  break;

	}

	delete pm;

      }
      
      if ( intact &&
	  (*j).second->getPixelStatus() != TimePixPixelMode::INTACT ) {
	//_timePixMap entry must be corrected to have PixelStatus = INTACT
	
	(*j).second->setPixelStatus( TimePixPixelMode::INTACT );
	intact = true;
	newintactstatus++;
	
      }
      
    }
  }
    
  else {

    //    throw( DataNotAvailableException)
    std::cout<<" TimePixMapListener::statusChanged: _timePixMap does not exist yet \n"
	     <<"always call TimePixModeMapListener before TimePixStatusMapListener"
	     <<std::endl;
  }

 std::cout<<" TimePixMapListener::statusChanged: "
	  <<"_timePixMap.size() = "<<_timePixMap.size()<<"\n"
	  <<" changed "<<newstatus<<" pixel to status dead/noisy : "
	  <<newdead<<"/"<<newnoisy<<"\n"
	  <<" changed "<<newintactstatus<<" pixel back to intact"
	  <<std::endl;

}
      
TimePixMap* TimePixMapListener::getMap() {

 
  return &_timePixMap;

}

TimePixMapListener::TimePixModeMapListener* TimePixMapListener::getModeMapListener(){

  return &_modeMapListener;
  
}

TimePixMapListener::TimePixStatusMapListener* TimePixMapListener::getStatusMapListener(){

  return &_statusMapListener;

}


}//namespace tpcconddata
