#ifndef PrintEventNumberProcessor_h
#define PrintEventNumberProcessor_h 1

// C++
#include <string>

// LCIO
#include "lcio.h"

// Marlin
#include "marlin/Processor.h"


namespace marlintpc{

    /* the following comment is tranfered to doxygen documentation
     * which begins with a second asterix, or in case of on line comment an extra third slash 
     */
    /**  Prints event numbers to the screen depending on the EveryNEvents variable.
     *
     *   @parameter EveryNEvents Print event number every n Events (default 100)
     * 
     *
     *  <h4>Input - Prerequisites</h4>
     *
     *  <h4>Output</h4> 
     * 
     * 
     * @author Simone Zimmermann, Martin Killenberg (Uni Bonn)
     */
    class PrintEventNumberProcessor : public marlin::Processor {
	
    public:
	
	/* This method will be called by the marlin package
	 * It returns a processor of the currend type (here PrintEventNumberProcessor)
	 */
	virtual Processor*  newProcessor() { return new PrintEventNumberProcessor ; }
	
	/* the default constructor
	 * here the processor parameters are registered to the marlin package
	 * other initialisation should be placed in the init method
	 */
	PrintEventNumberProcessor() ;
	
	/* Called at the beginning of the job before anything is read.
	 * Use to initialize the processor, e.g. book histograms
	 */
	virtual void init() ;
	
	/* Called for every run.
	 * in this processor it is not used
	 */
	virtual void processRunHeader( lcio::LCRunHeader* run ) ;
	
	/* Called for every event - the working horse.
	 * Here the real conversion take place
	 */
	virtual void processEvent( lcio::LCEvent * evt ) ; 
	
	/* This method is only called if the check flag is set (default) in the main processor
	 * (where the input files are given)
	 * It should be used to create check plots which are not needed in a complete analysis
	 * (where normaly the check flag is deactivated)
	 */
	virtual void check( lcio::LCEvent * evt ) ; 
	
	
	/* Called after data processing for clean up.
	 * e.g. saving control histogramms, delete created objects
	 * (which are not stored in a collection/event), etc.
	 */
	virtual void end() ;
	
	
    protected:
	
	/// variable how often to print the event number
	int _everyNEvents;

    } ;
    
}

#endif
    
    

